package com.ibm.websphere.query.callbacks;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.ibm.websphere.query.base.Condition;
import com.ibm.websphere.query.base.Constants;
import com.ibm.websphere.query.base.IAttribute;
import com.ibm.websphere.query.base.ICondition;
import com.ibm.websphere.query.base.IFrom;
import com.ibm.websphere.query.base.IJoin;
import com.ibm.websphere.query.base.ILeftOperand;
import com.ibm.websphere.query.base.IOperator;
import com.ibm.websphere.query.base.IOrderBy;
import com.ibm.websphere.query.base.IPredicate;
import com.ibm.websphere.query.base.IQuery;
import com.ibm.websphere.query.base.IRightOperand;
import com.ibm.websphere.query.base.ISelectQuery;
import com.ibm.websphere.query.base.ITable;
import com.ibm.websphere.query.base.IValue;
import com.ibm.websphere.query.base.Predicate;
import com.ibm.websphere.query.base.PredicateBase;
import com.ibm.websphere.query.base.QueryException;
import com.ibm.websphere.query.base.Value;

/**
 * Licensed Materials - Property of IBM. (C) COPYRIGHT International Business Machines Corp. 2002.
 */

/**
	<P>The CmQueryCallback class is a query builder callback for CM x-path queries. It provides
	attributes mapping and query syntax conversion for developers to build meaningful CM
	query strings.
 */
public class CmQueryCallback extends SqlSelectQueryCallback {

	private Map propertyColumnMap;
	private Map propertyTypeMap;

	public static final String CM_QUERY_STRINGSEP = "\"";
	public static final String CM_QUERY_ELEMENT = "element";
	public static final String CM_QUERY_LEFTSQUAREBRACE = "[";
	public static final String CM_QUERY_AND = "and";
   public static final String CM_QUERY_AND_QUERIES = "intersect";
	public static final String CM_QUERY_OR_CONDITIONS = "or";
	public static final String CM_QUERY_OR_QUERIES = "|";
	public static final String CM_QUERY_RIGHTSQUAREBRACE = "]";
	public static final String CM_QUERY_WILDCARD = "*";
	public static final String CM_OPERATOR_NOT = "!";

	public static final String CM_QUERY_LIKE = "icm:LIKE";
	public static final String CM_QUERY_ISNOTNULL = "isNotNull";
	public static final String CM_QUERY_NOT = "not";

	public static final String CM_QUERY_EQ = "=";
	public static final String CM_QUERY_NEQ = "!=";
	public static final String CM_QUERY_ORDERBYHEAD = "order by";
	public static final String CM_QUERY_ORDERBYASC = "ascending";
	public static final String CM_QUERY_ORDERBYDES = "descending";
	
	public static final String CM_QUERY_TRUE = "true()";
	public static final String CM_QUERY_FALSE = "false()";


   public static final String CM_ABSOLUTE_SEARCH = "/";
   public static final String CM_RECURSIVE_SEARCH = "//";
   public static final String CM_PATH_SEPARATOR = "/";
   public static final String CM_NT_PLACEHOLDER = "XYZCM_NT_PLACEXYZ";
   
   //CM Reserved Attributes
   public static final String CM_NODE_NAME = "nodeName";
   public static final String CM_NODE_PATH = "folder";
   public static final String CM_NODE_PARENT = "parent";
   public static final String CM_NODE_CHILD = "child";

	public static final Set CM_RESERVED_ATTR_SET = new HashSet();
	static {
		CM_RESERVED_ATTR_SET.add(CM_NODE_NAME);
		CM_RESERVED_ATTR_SET.add(CM_NODE_PATH);
	}

	public static final Map CM_OPERATOR_MAP = new HashMap();
	static {
		CM_OPERATOR_MAP.put(Constants.OPERATOR_EQ, CM_QUERY_EQ);
		CM_OPERATOR_MAP.put(Constants.OPERATOR_NEQ, CM_QUERY_NEQ);
		CM_OPERATOR_MAP.put(Constants.OPERATOR_IS, CM_QUERY_EQ);
		CM_OPERATOR_MAP.put(Constants.OPERATOR_ISN, CM_QUERY_NEQ);
		CM_OPERATOR_MAP.put(Constants.OPERATOR_ASC, CM_QUERY_ORDERBYASC);
		CM_OPERATOR_MAP.put(Constants.OPERATOR_DES, CM_QUERY_ORDERBYDES);
		CM_OPERATOR_MAP.put(Constants.OPERATOR_AND, CM_QUERY_AND);
		CM_OPERATOR_MAP.put(Constants.OPERATOR_OR, CM_QUERY_OR_CONDITIONS);
	}

	private static final List BINARY_OPERATIONS = new ArrayList(12);
	static {
		BINARY_OPERATIONS.add(Constants.OPERATOR_EQ);
		BINARY_OPERATIONS.add(Constants.OPERATOR_NEQ);
		BINARY_OPERATIONS.add(Constants.OPERATOR_GT);
		BINARY_OPERATIONS.add(Constants.OPERATOR_GEQ);
		BINARY_OPERATIONS.add(Constants.OPERATOR_LT);
		BINARY_OPERATIONS.add(Constants.OPERATOR_LEQ);
		BINARY_OPERATIONS.add(Constants.OPERATOR_IS);
		BINARY_OPERATIONS.add(Constants.OPERATOR_ISN);
		BINARY_OPERATIONS.add(Constants.OPERATOR_LIKE);
		BINARY_OPERATIONS.add(Constants.OPERATOR_NOT_LIKE);
		BINARY_OPERATIONS.add(Constants.OPERATOR_OR);
		BINARY_OPERATIONS.add(Constants.OPERATOR_AND);
	}

	/** 
	 * Constructor for CmQueryCallback
	 */
	public CmQueryCallback() {
		super(new HashMap(), new HashMap());
	}

	/**
	 * Constructor for CmQueryCallback.
	 * @param propertyColumnMap
	 */
	public CmQueryCallback(Map propertyColumnMap) {
		super(propertyColumnMap, new HashMap());
		this.propertyColumnMap = propertyColumnMap;
	}

	/**
	 * Constructor for CmQueryCallback.
	 * @param newNameMapping
	 * @param newTypeMapping
	 */
	public CmQueryCallback(Map propertyColumnMap, Map propertyTypeMap) {
		super(propertyColumnMap, propertyTypeMap);
		this.propertyColumnMap = propertyColumnMap;
		this.propertyTypeMap = propertyTypeMap;
	}

	public void buildAttribute(IAttribute attribute, StringBuffer sb) throws QueryException {
		String s = attribute.getName();
		if (s.equals(CM_NODE_CHILD) || s.equals(CM_NODE_NAME) || s.equals(CM_NODE_PARENT) || s.equals(CM_NODE_PATH)) 
			return;

		if ((nameMapping != null) && (nameMapping.containsKey(s))) {
			Object o = nameMapping.get(s);
			if (o instanceof IJoin) {
				IJoin join = (IJoin) o;
				IAttribute compareAttribute = join.getCompareAttribute();
				if (compareAttribute == null) {
					throw new QueryException("Cannot build join as attribute without compareAttribute", 1);
				} else {
					compareAttribute.buildString(this, sb);
				}
				s = "";
			} else {
				String tableName;
				ITable table = attribute.getTable();
				if (table != null) {
					tableName = table.getName();
				} else {
					tableName = getTableName();
				}
				if (tableName == null) {
					s = o.toString();
				} else {
					s = getTableAlias(tableName) + SQL_QUERY_DOT;
					s += o;
				}
			}
		}
      if (s.indexOf("@") == -1)
         s = "@" + CmPathUtil.xmlEncodePath(s);
      else
         s = s.substring(0, s.indexOf("@") + 1) + CmPathUtil.xmlEncodePath(s.substring(s.indexOf("@") + 1));

		sb.append(s);
	}

	/**
		<P>Returns the CM operator string for the given operator.
	 */
	public void buildOperator(IOperator operator, StringBuffer sb) throws QueryException {

		String operatorValue = (String) CM_OPERATOR_MAP.get(operator.getName());
		if (operatorValue != null)
			sb.append(operatorValue);
		else
			super.buildOperator(operator, sb);
		/*		super.buildOperator(operator, sb);
				String s = sb.toString();
		
				// For cm, change == to =   and leave everything else alone
				if (s.equals("=="))
					s = "=";
				if (s.equals("IS"))
					s = "=";
				if (s.equals("IS NOT"))
					s = "!=";
				if (s.equals("DESC"))
					s = "DESCENDING";
				else if (s.equals("ASC"))
					s = "ASCENDING";
		
				sb.setLength(0);
				sb.append(s);*/
	}

	/**
		<P>Returns the CM order by (sort by) string for the given select query.
	
		<P>This method uses the order by in the select query to generate the order by string in CM.
	 */
	public void buildOrderBy(ISelectQuery selectQuery, StringBuffer sb) throws QueryException
	{
	  IOrderBy o = selectQuery.getOrderBy();

	  if (o != null)
	  {
		sb.append(CM_QUERY_ORDERBYHEAD);
		o.buildString(this, sb);
	  }
	}

	/**
		<P>Returns the CM predicate string for the given predicate.
	
		<P>This method uses the predicate bases in the predicate to generate the predicate string in CM.
	 */
	public void buildPredicate(IPredicate predicate, StringBuffer sb) throws QueryException {
		int len = sb.length();
		int reservedAttrCount = 1;
		int returnedAttrCount = 0;
		//       String s2 = super.buildPredicate(predicate); System.out.println("$$$ super bp is:'"+s2+"'");
		if (predicate == null)
			return;
		IOperator o = predicate.getOperator();
		PredicateBase[] pb = predicate.getPredicateBases();

		for (int i = 0; i < pb.length; i++) {
			if (returnedAttrCount > 0) {
				if (!(pb[i] instanceof Condition)
					|| ((((Condition) pb[i]).getLeftOperand() instanceof IAttribute)
						&& !attributeIsReserved(((IAttribute) ((Condition) pb[i]).getLeftOperand()).getName()))) {
					//s = SQL_QUERY_LEFTPAREN + s + SQL_QUERY_RIGHTPAREN;
					sb.append(SQL_QUERY_SPACE);
					o.buildString(this, sb);
					sb.append(SQL_QUERY_SPACE);
				}
			}
			if (!(pb[i] instanceof Condition)
				|| ((((Condition) pb[i]).getLeftOperand() instanceof IAttribute)
					&& !attributeIsReserved(((IAttribute) ((Condition) pb[i]).getLeftOperand()).getName()))) {
				pb[i].buildString(this, sb);
				++returnedAttrCount;
			}

			//    System.out.println("i="+i+"   s="+s);
		}

		if (pb.length > reservedAttrCount) {
			sb.insert(len, SQL_QUERY_LEFTPAREN);
			sb.append(SQL_QUERY_RIGHTPAREN);
		}
	}

	public void buildFrom(IFrom from, StringBuffer sb) throws QueryException {
		ITable table = from.getTable();
      String nodetype;
		if (table != null)
			nodetype = table.getName();
		else
			nodetype = CM_QUERY_WILDCARD;

      int start = sb.indexOf(CM_NT_PLACEHOLDER);
      while (start != -1) {
         sb.replace(sb.indexOf(CM_NT_PLACEHOLDER), start + CM_NT_PLACEHOLDER.length(), CmPathUtil.xmlEncodePath(nodetype));
         start = sb.indexOf(CM_NT_PLACEHOLDER);
      }

	}

	/**
		<P>Returns the CM predicate string for the given query.
	
		<P>This method uses the predicate in the query to generate the predicate string in CM.
	 */
	public void buildPredicate(IQuery query, StringBuffer sb) throws QueryException {
		int len = sb.length();
		PredicateBase pb = query.getPredicate();

		if (pb != null) {
			pb.buildString(this, sb);
         if (sb.length() != len) {
            sb.insert(len, CM_QUERY_LEFTSQUAREBRACE);
			   sb.append(CM_QUERY_RIGHTSQUAREBRACE);
         }
		}
	}

	/**
	 <P>Returns the SQL select query string for the given select query.
	
	 <P>This method calls buildPredicate() and buildOrderBy() to generate the select query string
	 in SQL.
	 */
	public void buildSelectQuery(ISelectQuery selectQuery, StringBuffer sb) throws QueryException {
		buildPath(selectQuery, sb);
      buildFrom(selectQuery.getFrom(), sb);
		buildPredicate(selectQuery, sb);
		sb.append(SQL_QUERY_SPACE);
		buildOrderBy(selectQuery, sb);
		sb.append(SQL_QUERY_SPACE);
		buildLimit(selectQuery, sb);
	}

	private boolean buildPath(PredicateBase pb, StringBuffer sb, String operation) {
		if (pb instanceof Condition) {
			ILeftOperand l = ((Condition) pb).getLeftOperand();
			IOperator op = ((Condition) pb).getOperator();
			String attrName = "";
			if (l instanceof IAttribute)
				attrName = ((IAttribute) l).getName();
			if (attributeIsReserved(attrName)) {
				sb.append(operation);
				sb.append(convertAttribute(attrName, op.getName(), ((Value) ((Condition) pb).getRightOperands()[0]).getValueString()));
				return true;
			}
			return false;
		} else if (pb instanceof Predicate) {
			boolean firstFound = false;
			String op;
			if (((Predicate) pb).getOperator().getName().equals(Constants.OPERATOR_AND))
				op = " intersect ";
			else
				op = " | ";
			PredicateBase[] bases = ((Predicate) pb).getPredicateBases();
			StringBuffer sb2 = new StringBuffer();
			for (int i = 0; i < bases.length; ++i) {
				if (firstFound)
					firstFound = firstFound || buildPath(bases[i], sb2, op);
				else
					firstFound = firstFound || buildPath(bases[i], sb2, "");
			}
			if (sb2.length() > 0) {
				sb.append("(");
				sb.append(sb2);
				sb.append(")");
				return true;
			}
		}
		return false;
	}

	/**
		<P>Returns the CM path / element string for the given query.
	
		<P>This method uses the predicate and the from  in the query to 
	  generate the path / element string in CM.
	 */
	public void buildPath(IQuery query, StringBuffer sb) throws QueryException {
		HashMap attrs = new HashMap();
		StringBuffer sb2 = new StringBuffer();
		buildPath(query.getPredicate(), sb2, "");
		if (sb2.length() == 0) {
			sb.append(CM_RECURSIVE_SEARCH);
			sb.append(CM_QUERY_ELEMENT);
			sb.append("(*,");
			sb.append(CM_NT_PLACEHOLDER);
			sb.append(")");
		} else {
			sb.append(sb2);
		}
	}

/*		IRightOperand[] value = (IRightOperand[]) attrs.get(Constants.CM_NODE_PATH);
		if (value == null)
			sb.append("//"); //$NON-NLS-1$
		else {
			String path = ((Value) value[0]).getValueString();
			path = path + CmPathUtil.CM_PATH_SEPARATOR;
			path = CmPathUtil.normalizePath(path);
			sb.append(path);
		}
		if (value != null && value.length > 1) {
			for (int i = 1; i < value.length; ++i) {
				sb.append(SQL_QUERY_SPACE);
				sb.append(CM_QUERY_OR_QUERIES);
				sb.append(SQL_QUERY_SPACE);
				String path = ((Value) value[i]).getValueString();
				path = path + CmPathUtil.CM_PATH_SEPARATOR;
				path = CmPathUtil.normalizePath(path);
				sb.append(path);
			}
			sb.insert(0, SQL_QUERY_CONDITIONHEAD);
			sb.append(SQL_QUERY_CONDITIONTAIL);
		}
		sb.append(CM_QUERY_ELEMENT);
		sb.append(SQL_QUERY_CONDITIONHEAD);

		value = (IRightOperand[]) attrs.get(Constants.CM_NODE_NAME);
		if (value == null)
			sb.append(CM_QUERY_WILDCARD);
		else {
			if (value[0] instanceof IValue)
				sb.append(((Value) value[0]).getValueString());
			else
				value[0].buildString(this, sb);
		}

		sb.append(SQL_QUERY_COMMA);
		sb.append(SQL_QUERY_SPACE);

		IFrom type = query.getFrom();
		if (type == null)
			sb.append(CM_QUERY_WILDCARD);
		else
			type.buildString(this, sb);

		sb.append(SQL_QUERY_CONDITIONTAIL);
		sb.append(SQL_QUERY_SPACE);
	}  */

   private String buildPathString(String[] paths, String nodetype) {
//takes string[] of path elements
//returns element(str[0], *)/element(str[1],*)/element(str[n],nodetype);
      StringBuffer buf = new StringBuffer();
      for (int i = 0; i < paths.length; ++i) {
         buf.append(CM_QUERY_ELEMENT);
         buf.append("(");
         buf.append(CmPathUtil.xmlEncodePath(paths[i]));
         buf.append(",");
         if (i == paths.length - 1) {
            buf.append(CmPathUtil.xmlEncodePath(nodetype));
            buf.append(")");
         }
         else {
            buf.append("*");
            buf.append(")/");
         }
      }
      return buf.toString();
   }

   private String convertAttribute(String attribute, String operation, String value) {

      boolean useNodeName = false, useFolder = false, useParent = false, useChild = false;

      String prefix = CM_RECURSIVE_SEARCH;

      //single level node info only.  Rewrite needed for recursive support

      if (attribute.indexOf("nodeName") != -1)
         useNodeName = true;
      else if (attribute.indexOf("folder") != -1) {
         useFolder = true;
         if ("=".equals(operation))
            prefix = CM_ABSOLUTE_SEARCH;
      }
      if (attribute.indexOf("parent") != -1)
         useParent = true;
      else if (attribute.indexOf("child") != -1)
         useChild = true;
      if (value.startsWith(CM_RECURSIVE_SEARCH)) {
         value = value.substring(2);
         prefix = CM_RECURSIVE_SEARCH;
      }
      else if (value.startsWith(CM_ABSOLUTE_SEARCH)) {
         value = value.substring(1);
         prefix = CM_ABSOLUTE_SEARCH;
      }

      if (useFolder) {
         String[] pathList = CmPathUtil.getSegments(value);

         if (useParent)
            value = prefix + buildPathString(pathList, "*") + "/*/" + CM_QUERY_ELEMENT + "(*," + CM_NT_PLACEHOLDER + ")";
         else if (useChild)
            value = prefix + buildPathString(pathList, CM_NT_PLACEHOLDER) + "/*/..";
         else
            value = prefix + buildPathString(pathList, "*") + "/" + CM_QUERY_ELEMENT + "(*," + CM_NT_PLACEHOLDER + ")";
      }

      else if (useNodeName) {
         if (value.endsWith(CM_PATH_SEPARATOR))
            value = value.substring(0, value.length());
         value = CmPathUtil.xmlEncodePath(value);

         if (useParent)
            value = prefix + CM_QUERY_ELEMENT + "(" + value + ",*)/" + CM_QUERY_ELEMENT + "(*," + CM_NT_PLACEHOLDER + ")";
         else if (useChild)
            value = prefix + CM_QUERY_ELEMENT + "(*," + CM_NT_PLACEHOLDER + ")/" + CM_QUERY_ELEMENT + "(" + value + ",*)/..";
         else
            value = prefix + CM_QUERY_ELEMENT + "(" + value + "," + CM_NT_PLACEHOLDER + ")";
      }

      return value;
   }

	private boolean attributeIsReserved(String attribute) {
		Iterator iter = CM_RESERVED_ATTR_SET.iterator();
		while (iter.hasNext())
			if (attribute.indexOf((String)iter.next()) != -1)
				return true;
		return false;
	}

	public void buildValue(IValue value, StringBuffer sb) throws QueryException {
		String s = value.getValueString();
		if (s != null && s.indexOf("'") != -1) {
			s = makeQuerySafeValue(s);
		} else if (s == null || s.equalsIgnoreCase("NULL")) {
			s = "";
		}

		int dataType = value.getDataType();
		if (typeMapping != null && dataType == Constants.DATATYPE_UNKNOWN) {
			String propertyName = value.getPropertyName();
			if (propertyName != null) {
				Object o = typeMapping.get(propertyName);
				if (o != null && o instanceof Integer) {
					dataType = ((Integer) o).intValue();
				}
			}
		}

		switch (dataType) {
			case Constants.DATATYPE_BIGINT :
			case Constants.DATATYPE_DECIMAL :
			case Constants.DATATYPE_DOUBLE :
			case Constants.DATATYPE_FLOAT :
			case Constants.DATATYPE_TINYINT :
			case Constants.DATATYPE_NUMERIC :
			case Constants.DATATYPE_INTEGER :
			case Constants.DATATYPE_SMALLINT :
			case Constants.DATATYPE_REAL :
			case Constants.DATATYPE_UNKNOWN :
			case Constants.DATATYPE_NULL :
				break;
			case Constants.DATATYPE_DATE :
			case Constants.DATATYPE_TIME :
			case Constants.DATATYPE_TIMESTAMP :
				if (XPathDateFormatter.getInstance().isDate(s))
					s = XPathDateFormatter.getInstance().formatFromDate(s);
			case Constants.DATATYPE_CHAR :
			case Constants.DATATYPE_LONGVARCHAR :
			case Constants.DATATYPE_VARCHAR :
				s = CM_QUERY_STRINGSEP + s + CM_QUERY_STRINGSEP;
				break;
			case Constants.DATATYPE_BINARY :
				if (s.equalsIgnoreCase("true") || s.equalsIgnoreCase("on") || s.equalsIgnoreCase("yes"))
					s = CM_QUERY_TRUE;
				else
					s = CM_QUERY_FALSE;
				break;
			default :
				throw new QueryException("Data type not supported.", 3);
		}
		sb.append(s);
	}

	public String makeQuerySafeValue(String myValueString) {
		java.util.StringTokenizer st = new java.util.StringTokenizer(myValueString, "'", true);
		StringBuffer sb = new StringBuffer(myValueString.length() * 2);

		while (st.hasMoreTokens()) {
			String s = st.nextToken();

			if (s.equals(CM_QUERY_STRINGSEP)) {
				sb.append(s + s);
			} else {
				sb.append(s);
			}
		}
		return sb.toString();
	}

   private void prefixAttributes(PredicateBase base, String type) {
      if (base instanceof Predicate) {
         PredicateBase[] bases = ((Predicate)base).getPredicateBases();
         for (int i = 0; i < bases.length; ++i)
            prefixAttributes(bases[i], type);
      }
      else if (base instanceof Condition) {
         ILeftOperand operand = ((Condition)base).getLeftOperand();
         if (operand instanceof IAttribute) {
            String name = ((IAttribute)operand).getName();
            String prefix;
            if (type.equals(CM_NODE_PARENT))
               prefix = "../";
            else
               prefix = "./*/";
            if (name.startsWith("./") && name.length() > 2 && type.equals(CM_NODE_CHILD))
               name = name.substring(2);
            if(name.indexOf("@") == -1)
               name = prefix + "@" + name;
            else
               name = prefix + name;
            ((IAttribute)operand).setName(name);
         }
      }
   }

	/**
		<P>Returns the SQL condition string for the given condition.
	
		<P>This method defines string generation for conditions with all possible operators supported
		in SQL.	Note that the left operand is checked for multiplicity first because single-value
		operators have unique meanings for SQL if the left operand is a multi-value attribute.
	 */
	public void buildCondition(ICondition condition, StringBuffer sb) throws QueryException {
		IOperator o = condition.getOperator();
		ILeftOperand l = condition.getLeftOperand();
		IRightOperand[] r = condition.getRightOperands();

		boolean isLeftOperandMVAttribute = false;
		boolean isRightOperandsNull = false;
		if (l instanceof IAttribute) {
			isLeftOperandMVAttribute = isMultiValueAttribute((IAttribute) l);
			if (isLeftOperandMVAttribute) {
				Set multiValuePropertyNames = (Set) objectMapping.get(MULTIVALUE_PROPERTY_NAMES);
				if (multiValuePropertyNames == null) {
					multiValuePropertyNames = new HashSet();
					objectMapping.put(MULTIVALUE_PROPERTY_NAMES, multiValuePropertyNames);
				}
				multiValuePropertyNames.add(((IAttribute) l).getName());
			}
			String name = ((IAttribute)l).getName();
			if (name.equals(CM_NODE_PARENT) ||
			    name.equals(CM_NODE_CHILD)) {
				if (r.length > 0 && r[0] instanceof PredicateBase) {
               prefixAttributes((PredicateBase)r[0], name);
					r[0].buildString(this, sb);
					return;
				}
			}
		}
		sb.append(SQL_QUERY_CONDITIONHEAD);
		if (r.length == 0 || r[0] == null || (r[0] instanceof Value && (((Value) r[0]).getValueString() == null || ((Value) r[0]).getValueString().equals("NULL")) || ((Value) r[0]).getDataType() == Constants.DATATYPE_NULL)) {
			isRightOperandsNull = true;
		}

		String operatorName = o.getName();

		if (isRightOperandsNull) {
			if (operatorName.equals(Constants.OPERATOR_IS) || operatorName.equals(Constants.OPERATOR_EQ) || operatorName.equals(Constants.OPERATOR_LIKE) || operatorName.equals(Constants.OPERATOR_IN)) {
				sb.append(CM_QUERY_NOT);
				sb.append(SQL_QUERY_LEFTPAREN);
				sb.append(CM_QUERY_ISNOTNULL);
				sb.append(SQL_QUERY_LEFTPAREN);
				l.buildString(this, sb);
				sb.append(SQL_QUERY_RIGHTPAREN);
				sb.append(SQL_QUERY_RIGHTPAREN);
			} else if (operatorName.equals(Constants.OPERATOR_ISN) || operatorName.equals(Constants.OPERATOR_NEQ) || operatorName.equals(Constants.OPERATOR_NOT_LIKE)){
				sb.append(CM_QUERY_ISNOTNULL);
				sb.append(SQL_QUERY_LEFTPAREN);
				l.buildString(this, sb);
				sb.append(SQL_QUERY_RIGHTPAREN);
			} else {
				throw new QueryException("right operand is null for invalid operator " + operatorName);
			}
		} else if (operatorName.equals(Constants.OPERATOR_LIKE)) {
			sb.append(CM_QUERY_LIKE);
			sb.append(SQL_QUERY_LEFTPAREN);
			l.buildString(this, sb);
			
			sb.append(SQL_QUERY_COMMA);
			sb.append(SQL_QUERY_SPACE);
			if (r[0] instanceof Value)
			{
				Value v = (Value) r[0];
				int dt = v.getDataType();
				if ((dt == Constants.DATATYPE_CHAR) || (dt == Constants.DATATYPE_LONGVARCHAR) || (dt == Constants.DATATYPE_VARCHAR)) {
					String vs = v.getValueString();
	
					if (vs.indexOf(SQL_QUERY_WILDCARD) == -1) {
						vs = CM_QUERY_STRINGSEP + SQL_QUERY_WILDCARD + vs + SQL_QUERY_WILDCARD + CM_QUERY_STRINGSEP;
					} else {
						vs = CM_QUERY_STRINGSEP + vs + CM_QUERY_STRINGSEP;
					}
					
					sb.append(vs);
				} else {
					throw new QueryException("Invalid data type for operator '" + operatorName + "'.", 1);
				}
			} else {
			  throw new QueryException("Operator '" + operatorName + "' not supported in this context.", 1);
			}
			sb.append(SQL_QUERY_RIGHTPAREN);
		} else if (isBinaryOperation(o)) {
			if (isLeftOperandMVAttribute) {
				buildSMVCondition(condition, sb);
			} else {
				buildConditionForBinaryOperation(condition, sb);
			}
		} else if (operatorName.equals(Constants.OPERATOR_BTW)) {
			if (isLeftOperandMVAttribute) {
				buildSMVCondition(condition, sb);
			} else {
				// left
				l.buildString(this, sb);
				sb.append(SQL_QUERY_SPACE);
				// >=
				Constants.OP_GEQ.buildString(this, sb);
				sb.append(SQL_QUERY_SPACE);
				// r0
				r[0].buildString(this, sb);
				sb.append(SQL_QUERY_SPACE);
				// and
				Constants.OP_AND.buildString(this, sb);
				sb.append(SQL_QUERY_SPACE);
				// left
				l.buildString(this, sb);
				sb.append(SQL_QUERY_SPACE);
				// <=
				Constants.OP_LEQ.buildString(this, sb);
				sb.append(SQL_QUERY_SPACE);
				// r1
				r[1].buildString(this, sb);
			}
		// for isequal where the right hand side is a specific day, it should be an inbetween in XPATH
		// TODO do something similar for not equals
		} else if (r != null
			&& r.length > 0
			&& (r[0] instanceof Value && ((Value) r[0]).getDataType() == Constants.DATATYPE_DATE)
			&& condition instanceof Condition && (operatorName.equals(Constants.OPERATOR_EQ) || operatorName.equals(Constants.OPERATOR_NEQ))
			&& XPathDateFormatter.getInstance().isDate(((Value) r[0]).getValueString())) {

			if (operatorName.equals(Constants.OPERATOR_EQ)) {
				o = Constants.OP_BTW;
				r =
					new IRightOperand[] {
						r[0],
						new Value(XPathDateFormatter.getInstance().formatFromDateToEndOfDay(((Value) r[0]).getValueString()), Constants.DATATYPE_DATE)};
				condition = new Condition(l, o, r);
				buildCondition(condition, sb);
			} else {
				sb.append(CM_QUERY_NOT);
				sb.append(SQL_QUERY_LEFTPAREN);
				o = Constants.OP_BTW;
				r =
					new IRightOperand[] {
						r[0],
						new Value(XPathDateFormatter.getInstance().formatFromDateToEndOfDay(((Value) r[0]).getValueString()), Constants.DATATYPE_DATE)};
				condition = new Condition(l, o, r);
				buildCondition(condition, sb);
				sb.append(SQL_QUERY_RIGHTPAREN);
			}
		} else {
			super.buildCondition(condition, sb);
		}
		sb.append(SQL_QUERY_CONDITIONTAIL);
	}

	protected boolean isBinaryOperation(IOperator o) {
		return BINARY_OPERATIONS.contains(o.getName());
	}

	protected void buildConditionForBinaryOperation(ICondition condition, StringBuffer sb) throws QueryException {
		IOperator o = condition.getOperator();
		ILeftOperand l = condition.getLeftOperand();
		IRightOperand[] r = condition.getRightOperands();

		l.buildString(this, sb);
		sb.append(SQL_QUERY_SPACE);
		o.buildString(this, sb);
		sb.append(SQL_QUERY_SPACE);
		if (r.length > 1)
			sb.append(SQL_QUERY_LEFTPAREN);
		for (int i = 0; i < r.length; i++) {
			if (i > 0) {
				sb.append(SQL_QUERY_COMMA);
				sb.append(SQL_QUERY_SPACE);
			}
			r[i].buildString(this, sb);
		}
		if (r.length > 1)
			sb.append(SQL_QUERY_RIGHTPAREN);

	}

}
