package com.ibm.websphere.query.callbacks;

import java.util.Hashtable;
import java.util.Map;

import com.ibm.websphere.query.base.Attribute;
import com.ibm.websphere.query.base.Constants;
import com.ibm.websphere.query.base.IArithmeticCondition;
import com.ibm.websphere.query.base.IAttribute;
import com.ibm.websphere.query.base.ICondition;
import com.ibm.websphere.query.base.IFunction;
import com.ibm.websphere.query.base.ILeftOperand;
import com.ibm.websphere.query.base.ILimit;
import com.ibm.websphere.query.base.IOperator;
import com.ibm.websphere.query.base.IOrderBy;
import com.ibm.websphere.query.base.IOrderByElement;
import com.ibm.websphere.query.base.IOrderByExpression;
import com.ibm.websphere.query.base.IPredicate;
import com.ibm.websphere.query.base.IQuery;
import com.ibm.websphere.query.base.IRightOperand;
import com.ibm.websphere.query.base.ISelectQuery;
import com.ibm.websphere.query.base.IValue;
import com.ibm.websphere.query.base.PredicateBase;
import com.ibm.websphere.query.base.QueryException;
import com.ibm.websphere.query.base.Value;

/**
 * Licensed Materials - Property of IBM. (C) COPYRIGHT International Business Machines Corp. 2002.
 */

/**
	<P>The LdapSelectQueryCallback class is a query builder callback for LDAP. It provides 
	attributes mapping and query syntax conversion for developers to build meaningful LDAP 
	query strings.
		
	<P>It implments the ISelectQueryCallback interface so it can be used whenever an ISelectQueryCallback
	is needed.
	
	<P>This class has a hash table that contains SQL attributes mapping. When building an 
	attribute string, the hash table is looked up to perform an attribute conversion.
 */
public class LdapSelectQueryCallback extends SelectQueryCallbackAdapter implements com.ibm.websphere.query.base.ISelectQueryCallback {

    public static final String LDAP_QUERY_LEFTPAREN = "(";
    public static final String LDAP_QUERY_NOT = "!";
    public static final String LDAP_QUERY_RIGHTPAREN = ")";

    public static final String LDAP_QUERY_PREDICATEHEAD = "(";
    public static final String LDAP_QUERY_PREDICATETAIL = ")";
    public static final String LDAP_QUERY_CONDITIONHEAD = "(";
    public static final String LDAP_QUERY_CONDITIONTAIL = ")";

    public static final String LDAP_QUERY_AND = "&";

    public static final String LDAP_QUERY_OR = "|";
    public static final String LDAP_QUERY_EQ = "=";
    public static final String LDAP_QUERY_LEQ = "<=";
    public static final String LDAP_QUERY_GEQ = ">=";

    private Map nameMapping = null;
    protected Map typeMapping = null;
    /**
    	Creates a new LdapSelectQueryCallback instance, default constructor.
     */
    public LdapSelectQueryCallback() {
        super();
    }
    /**
    	Creates a new LdapSelectQueryCallback instance with the given attribute mapping.
     */
    public LdapSelectQueryCallback(Hashtable ht) {
        super();
        nameMapping = ht;
    }
    /**
    	Creates a new LdapSelectQueryCallback instance with the given attribute and type mappings.
     */
    public LdapSelectQueryCallback(Map newNameMapping, Map newTypeMapping) {
        super();
        nameMapping = newNameMapping;
        typeMapping = newTypeMapping;
    }
    /**
    	<P>Returns the LDAP attribute string for the given attribute. 
     */
    public String buildAttribute(IAttribute attribute) throws QueryException {
        String s = attribute.getName();

        if ((nameMapping != null) && (nameMapping.containsKey(s)))
            s = (String) nameMapping.get(s);

        return s;
    }
    /**
    	<P>Returns the LDAP condition string for the given condition. 
    	
    	<P>This method defines string generation for conditions with all possible operators supported
    	in LDAP.
     */
    public String buildCondition(ICondition condition) throws QueryException {
        String s = LDAP_QUERY_CONDITIONHEAD;

        IOperator o = condition.getOperator();
        ILeftOperand l = condition.getLeftOperand();
        IRightOperand[] r = condition.getRightOperands();

        boolean isRightOperandsNull = false;
        if (r.length == 0 || r[0] == null) {
            isRightOperandsNull = true;
        }
        else if (r[0] instanceof Value) {
            isRightOperandsNull = ((Value) r[0]).getValueString() == null;
        }

        String operatorName = o.getName();

        if (operatorName.equals(Constants.OPERATOR_GEQ) || operatorName.equals(Constants.OPERATOR_LEQ) || operatorName.equals(Constants.OPERATOR_IS)) {
            s = s + l.buildString(this);
            s = s + o.buildString(this);
            s = s + r[0].buildString(this);
        }
        else if (operatorName.equals(Constants.OPERATOR_EQ)) {
            if (isRightOperandsNull) {
                s = s + LDAP_QUERY_NOT;
                s = s + LDAP_QUERY_LEFTPAREN;
                s = s + l.buildString(this);
                s = s + o.buildString(this);
                s = s + LDAP_QUERY_WILDCARD;
                s = s + LDAP_QUERY_RIGHTPAREN;
            }
            else {
                s = s + l.buildString(this);
                s = s + o.buildString(this);
                s = s + r[0].buildString(this);
            }
        }
        else if (operatorName.equals(Constants.OPERATOR_BTW)) {

            String s1 = l.buildString(this);
            s1 = s1 + LDAP_QUERY_GEQ;
            s1 = s1 + r[0].buildString(this);
            s1 = s1 + LDAP_QUERY_RIGHTPAREN;
            s1 = LDAP_QUERY_LEFTPAREN + s1;

            String s2 = l.buildString(this);
            s2 = s2 + LDAP_QUERY_LEQ;
            s2 = s2 + r[1].buildString(this);
            s2 = s2 + LDAP_QUERY_RIGHTPAREN;
            s2 = LDAP_QUERY_LEFTPAREN + s2;

            s = s + LDAP_QUERY_AND + s1 + s2;

        }
        else if (operatorName.equals(Constants.OPERATOR_IN)) {

            String s0 = l.buildString(this);
            String s1 = "";
            for (int i = 0; i < r.length; i++) {
                s1 = s1 + LDAP_QUERY_LEFTPAREN + s0 + LDAP_QUERY_EQ + r[i].buildString(this) + LDAP_QUERY_RIGHTPAREN;
            }
            s = s + LDAP_QUERY_OR + s1;

        }
        else if (operatorName.equals(Constants.OPERATOR_NEQ)) {
            if (isRightOperandsNull) {
                s = s + l.buildString(this);
                s = s + LDAP_QUERY_EQ;
                s = s + LDAP_QUERY_WILDCARD;
            }
            else {
                s = s + l.buildString(this);
                s = s + LDAP_QUERY_EQ;
                s = s + r[0].buildString(this);
                s = LDAP_QUERY_NOT + s;
                s = LDAP_QUERY_LEFTPAREN + s;
                s = s + LDAP_QUERY_RIGHTPAREN;
            }
        }
        else if (operatorName.equals(Constants.OPERATOR_GT)) {

            String s1 = l.buildString(this);
            s1 = s1 + LDAP_QUERY_GEQ;
            s1 = s1 + r[0].buildString(this);
            s1 = s1 + LDAP_QUERY_RIGHTPAREN;
            s1 = LDAP_QUERY_LEFTPAREN + s1;

            String s2 = l.buildString(this);
            s2 = s2 + LDAP_QUERY_EQ;
            s2 = s2 + r[0].buildString(this);
            s2 = s2 + LDAP_QUERY_RIGHTPAREN;
            s2 = LDAP_QUERY_LEFTPAREN + s2;

            s2 = LDAP_QUERY_NOT + s2;
            s2 = LDAP_QUERY_LEFTPAREN + s2;
            s2 = s2 + LDAP_QUERY_RIGHTPAREN;

            s = s + LDAP_QUERY_AND + s1 + s2;
        }
        else if (operatorName.equals(Constants.OPERATOR_LT)) {

            String s1 = l.buildString(this);
            s1 = s1 + LDAP_QUERY_LEQ;
            s1 = s1 + r[0].buildString(this);
            s1 = s1 + LDAP_QUERY_RIGHTPAREN;
            s1 = LDAP_QUERY_LEFTPAREN + s1;

            String s2 = l.buildString(this);
            s2 = s2 + LDAP_QUERY_EQ;
            s2 = s2 + r[0].buildString(this);
            s2 = s2 + LDAP_QUERY_RIGHTPAREN;
            s2 = LDAP_QUERY_LEFTPAREN + s2;

            s2 = LDAP_QUERY_NOT + s2;
            s2 = LDAP_QUERY_LEFTPAREN + s2;
            s2 = s2 + LDAP_QUERY_RIGHTPAREN;

            s = s + LDAP_QUERY_AND + s1 + s2;
        }
        else if (operatorName.equals(Constants.OPERATOR_LIKE)) {
            if ((l instanceof Attribute) && (r[0] instanceof Value)) {
                Value v = (Value) r[0];
                int dt = v.getDataType();

                if ((dt == Constants.DATATYPE_CHAR) || (dt == Constants.DATATYPE_LONGVARCHAR) || (dt == Constants.DATATYPE_VARCHAR)) {
                    String vs = v.getValueString();

                    if (vs.indexOf(LDAP_QUERY_WILDCARD) == -1) {
                        vs = LDAP_QUERY_WILDCARD + vs + LDAP_QUERY_WILDCARD;
                    }
                    v.setValueString(vs);

                    s = s + l.buildString(this);
                    s = s + o.buildString(this);
                    s = s + r[0].buildString(this);
                }
                else {
                    throw new QueryException("Invalid data type for operator '" + operatorName + "'.", 1);
                }
            }
            else {
                throw new QueryException("Operator '" + operatorName + "' not supported in this context.", 1);
            }
        }
        else if (operatorName.equals(Constants.OPERATOR_NOT_LIKE)) {
            if ((l instanceof Attribute) && (r[0] instanceof Value)) {
                Value v = (Value) r[0];
                int dt = v.getDataType();

                if ((dt == Constants.DATATYPE_CHAR) || (dt == Constants.DATATYPE_LONGVARCHAR) || (dt == Constants.DATATYPE_VARCHAR)) {
                    String vs = v.getValueString();

                    if (vs.indexOf(LDAP_QUERY_WILDCARD) == -1) {
                        vs = LDAP_QUERY_WILDCARD + vs + LDAP_QUERY_WILDCARD;
                    }
                    v.setValueString(vs);

                    s = s + l.buildString(this);
                    s = s + LDAP_QUERY_LIKE;
                    s = s + r[0].buildString(this);
                    s = LDAP_QUERY_NOT + s;
                    s = LDAP_QUERY_LEFTPAREN + s;
                    s = s + LDAP_QUERY_RIGHTPAREN;
                }
                else {
                    throw new QueryException("Invalid data type for operator '" + operatorName + "'.", 1);
                }
            }
            else {
                throw new QueryException("Operator '" + operatorName + "' not supported in this context.", 1);
            }
        }
        else {
            throw new QueryException("Operator '" + operatorName + "' not supported.", 1);
        }

        s = s + LDAP_QUERY_CONDITIONTAIL;
        return s;
    }
    /**
    	<P>Throws an exception because LDAP does not support functions.
     */
    public String buildFunction(IFunction function) throws QueryException {
        throw new QueryException("Function '" + function.getName() + "' not supported.", 2);
    }
    /**
    	<P>Returns the LDAP operator string for the given operator. 
    
    	<P>This method defines string generation for all possible operators supported in LDAP.
     */
    public String buildOperator(IOperator operator) throws QueryException {
        String s = "";

        String operatorName = operator.getName();

        if (operatorName.equals(Constants.OPERATOR_AND)) {
            s = LDAP_QUERY_AND;
        }
        else if (operatorName.equals(Constants.OPERATOR_OR)) {
            s = LDAP_QUERY_OR;
        }
        else if (operatorName.equals(Constants.OPERATOR_EQ)) {
            s = LDAP_QUERY_EQ;
        }
        else if (operatorName.equals(Constants.OPERATOR_GEQ)) {
            s = LDAP_QUERY_GEQ;
        }
        else if (operatorName.equals(Constants.OPERATOR_LEQ)) {
            s = LDAP_QUERY_LEQ;
        }
        else if (operatorName.equals(Constants.OPERATOR_ASC)) {
            s = LDAP_QUERY_ORDERBYASC;
        }
        else if (operatorName.equals(Constants.OPERATOR_DES)) {
            s = LDAP_QUERY_ORDERBYDES;
        }
        else if (operatorName.equals(Constants.OPERATOR_LIKE)) {
            s = LDAP_QUERY_LIKE;
        }
        else {
            throw new QueryException("Operator '" + operatorName + "' not supported.", 1);
        }

        return s;
    }
    /**
    	<P>This method iterates through the order by expressions in the order by object to generate
    	the order by string in LDAP.
     */
    public String buildOrderBy(IOrderBy orderBy) throws QueryException {

        IOrderByExpression[] oa = orderBy.getOrderByExpressions();
        if (sortStrings == null) {
            sortStrings = new String[(oa.length) * 2];
        }
        for (int i = 0; i < oa.length; i++) {
            oa[i].buildString(this);
        }
        return null;
    }
    /**
    	<P>This method uses the order by in the select query to generate the order by string in LDAP.
     */
    public String buildOrderBy(ISelectQuery selectQuery) throws QueryException {
        IOrderBy o = selectQuery.getOrderBy();
        if (o != null)
            o.buildString(this);
        return null;
    }
    /**
    	<P>This method uses the order by element and operator in the order by expression to generate
    	the order by expression strings as an array of strings. The results are stored as an instance 
    	variable so the builder callback users can use them to construct the final LDAP query string.
     */
    public String buildOrderByExpression(IOrderByExpression orderByExpression) throws QueryException {

        IOperator o = orderByExpression.getOperator();
        IOrderByElement oe = orderByExpression.getOrderByElement();

        if (sortStrings != null) {
            sortStrings[nbSortStrings++] = oe.buildString(this);
            sortStrings[nbSortStrings++] = o.buildString(this);
        }

        return null;
    }
    /**
    	<P>Returns the LDAP predicate string for the given predicate. 
    
    	<P>This method uses the predicate bases in the predicate to generate the predicate string in 
    	LDAP.
     */
    public String buildPredicate(IPredicate predicate) throws QueryException {
        String s = LDAP_QUERY_PREDICATEHEAD;

        IOperator o = predicate.getOperator();
        PredicateBase[] pb = predicate.getPredicateBases();

        for (int i = 0; i < pb.length; i++) {
            if (i == 0)
                s = s + o.buildString(this);
            s = s + pb[i].buildString(this);
        }
        s = s + LDAP_QUERY_PREDICATETAIL;

        return s;
    }
    /**
    	<P>Returns the LDAP predicate string for the given query. 
    
    	<P>This method uses the predicate in the query to generate the predicate string in LDAP.
     */
    public String buildPredicate(IQuery query) throws QueryException {
        String s = "";
        PredicateBase pb = query.getPredicate();

        if (pb != null) {
            s = pb.buildString(this);
        }
        return s;
    }
    /**
    	<P>Returns the LDAP query string for the given query. 
    
    	<P>This method calls buildPredicate() to generate the query string in LDAP.
     */
    public String buildQuery(IQuery query) throws QueryException {
        return buildPredicate(query);
    }
    /**
    	<P>Returns the LDAP select query string for the given select query. 
    
    	<P>This method calls buildPredicate() to generate the select query string in LDAP. It also 
    	calls buildOrderBy() to generate the string array for order by as an instance variable.
     */
    public String buildSelectQuery(ISelectQuery selectQuery) throws QueryException {
        buildOrderBy(selectQuery);
        return buildPredicate(selectQuery);
    }
    /**
    	<P>Returns the LDAP value string for the given value.
    
    	<P>This method generates the value string in LDAP based on the data type.
     */
    public String buildValue(IValue value) throws QueryException {
        String s = buildValueString(value.getValueString());

        int dataType = value.getDataType();
        if (typeMapping != null && dataType == Constants.DATATYPE_UNKNOWN) {
            String propertyName = value.getPropertyName();
            if (propertyName != null) {
                Object o = typeMapping.get(propertyName);
                if (o != null && o instanceof Integer) {
                    dataType = ((Integer) o).intValue();
                }
            }
        }

        switch (dataType) {
            case Constants.DATATYPE_UNKNOWN :
            case Constants.DATATYPE_NULL :
                break;
            case Constants.DATATYPE_CHAR :
            case Constants.DATATYPE_LONGVARCHAR :
            case Constants.DATATYPE_VARCHAR :
                break;
            default :
                throw new QueryException("Data type not supported.", 3);
        }

        return s;
    }
    /**
    	Returns the attribute mapping hash table of the builder callback.
     */
    public java.util.Map getNameMapping() {
        return nameMapping;
    }
    /**
    	Sets the attribute mapping hash table of the builder callback.
     */
    public void setNameMapping(java.util.Hashtable newNameMapping) {
        nameMapping = newNameMapping;
    }
    public static final String LDAP_QUERY_LIKE = "=";
    public static final String LDAP_QUERY_ORDERBYASC = "ASC";
    public static final String LDAP_QUERY_ORDERBYDES = "DESC";
    public static final String LDAP_QUERY_WILDCARD = "*";
    protected int nbSortStrings = 0;
    protected String[] sortStrings = null;
    /**
    	<P>Returns the LDAP query safe value string for the given string.
     */
    protected String buildValueString(String valueString) throws QueryException {
        String s = valueString;
        return s;
    } /**
    	Returns the sort strings of the builder callback.
    */
    public String[] getSortStrings() {
        return sortStrings;
    } /**
    * @see ISelectQueryCallback#buildArithmeticCondition(IArithmeticCondition)
    */
    public String buildArithmeticCondition(IArithmeticCondition arithmeticCondition) throws QueryException {
        // Not Supported on LDAP because the only datatypes supported are Strings, and we currently do not
        // do arithmetic operations on Strings
        return null;
    }

	public long getLimit(ISelectQuery selectQuery) throws QueryException {
		ILimit limit = selectQuery.getLimit();
		if (limit != null)
			return limit.getLimitValue();
		return 0;
	}

    /**
     * @see ISelectQueryCallback#buildLimit(ILimit)
     */
    public String buildLimit(ILimit limit) throws QueryException {
        return new Integer(limit.getLimitValue()).toString();
    }

    /**
     * @see ISelectQueryCallback#buildLimit(ISelectQuery)
     */
    public String buildLimit(ISelectQuery selectQuery) throws QueryException {
        String limitString = "";
        ILimit limit = selectQuery.getLimit();
        if (limit != null) {
        	//may need to add some strings here depending on what the resource is expecting
            limitString = limit.buildString(this);
        }
        return limitString;
    }

}