/*********************************************************************
 * Licensed Materials - Property of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.serialization;

import java.util.Map;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import com.ibm.websphere.query.base.ArithmeticCondition;
import com.ibm.websphere.query.base.Attribute;
import com.ibm.websphere.query.base.Condition;
import com.ibm.websphere.query.base.Constants;
import com.ibm.websphere.query.base.Function;
import com.ibm.websphere.query.base.IArithmeticCondition;
import com.ibm.websphere.query.base.IFunction;
import com.ibm.websphere.query.base.IPredicate;
import com.ibm.websphere.query.base.ISelectQuery;
import com.ibm.websphere.query.base.Limit;
import com.ibm.websphere.query.base.Operator;
import com.ibm.websphere.query.base.OrderBy;
import com.ibm.websphere.query.base.OrderByExpression;
import com.ibm.websphere.query.base.Predicate;
import com.ibm.websphere.query.base.Value;

/**
 * This parser unmarshalls a SelectQuery object from XML.
 * 
 * The method naming convention is as follows:
 * 
 *   All of the handle* methods are used to handle the various states.
 * 
 *   The handle prefix can be followed by one or two element names.
 *     Two element case (i.e. handleQueryCondition):
 *       This method is invoked when we are inside of the first element (Query)
 *       and encounter the second element (Condition) for the first time.  The first
 *       time we encounter the element, it needs to be created, the state needs to be
 *       set and the object needs to be set on the parent object (Query in this case).
 *     One element case (i.e. handleFunction):
 *       This method is invoked when we are inside of this element.  In this example,
 *       handleCondition would be called when we are already inside of a condition element,
 *       regardless of the parent element.  For instance, functions can be inside of conditions,
 *       arithmetic conditions, order by expressions, and other functions, but the handling of
 *       the function element is independent from the parent in all cases.  The function will
 *       be added to the parent in the two element case method (handleConditionFunction, etc.) 
 *       and then the handleFunction method will simply populate the object.  The endElement calls 
 *       will be used to check the parent and restore state.
 *       
 *     You will notice that function and arithmetic conditions are exceptions to the rule.
 *     Since both of these elements can contain each other, the state is handled in a different way.
 *     If a function is encountered within an arithmetic condition (or vice versa) the state will be
 *     set so that we are no long in an arithmetic condition and we are now in a function.  Although
 *     this is technically an inaccurate state (because we are still within an arithmetic expression)
 *     this is required to actually reach an end point (the handleFunction or handleArithmeticCondition
 *     method).  The stack is used to obtain the parent object and maintain the correct hierarchy.
 */
public class XMLSelectQueryContentHandler implements ContentHandler
{

  /* The select query */
  private ISelectQuery selectQuery = null;

  /* State variables */
  private boolean inQuery = false;
  private boolean inArithmeticCondition = false;
  private boolean inFunction = false;
  private boolean inCondition = false;
  private boolean foundConditionLeftOperand = false;
  private boolean inOrderBy = false;
  private boolean inOrderByExpression = false;
  private boolean inPredicate = false;
  private boolean foundOperator = false;
  private boolean inCriteria = false;

  /* Object stack */
  private Stack queryStack = null;

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#characters(char[], int, int)
   */
  public void characters(char[] ch, int start, int length) throws SAXException
  {
    // Ignore the characters
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#endDocument()
   */
  public void endDocument() throws SAXException
  {
    queryStack = null;
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
   */
  public void endElement(String namespaceURI, String localName, String qName) throws SAXException
  {
    // Figure out which element we are leaving
    if (localName.equals(XMLConstants.FUNCTION_ELEMENT))
    {
      // Set the state
      inFunction = false;

      // Remove this object from the stack
      queryStack.pop();

      // Check the parent to correct set the state
      Object parent = queryStack.peek();
      if (parent instanceof IArithmeticCondition)
      {
        inArithmeticCondition = true;
      }
      else if (parent instanceof IFunction)
      {
        inFunction = true;
      }
    }
    else if (localName.equals(XMLConstants.ARITHMETIC_CONDITION_ELEMENT))
    {
      // Set the state
      inArithmeticCondition = false;

      // Remove this object from the stack
      queryStack.pop();

      // Check the parent to correct set the state
      Object parent = queryStack.peek();
      if (parent instanceof IArithmeticCondition)
      {
        inArithmeticCondition = true;
      }
      else if (parent instanceof IFunction)
      {
        inFunction = true;
      }
    }
    else if (localName.equals(XMLConstants.ORDER_BY_EXPRESSION_ELEMENT))
    {
      // Set the state
      inOrderByExpression = false;
      foundOperator = false;

      // Remove this object from the stack
      queryStack.pop();
    }
    else if (localName.equals(XMLConstants.ORDER_BY_ELEMENT))
    {
      // Set the state
      inOrderBy = false;

      // Remove this object from the stack
      queryStack.pop();
    }
    else if (localName.equals(XMLConstants.CONDITION_ELEMENT))
    {
      // Set the state
      inCondition = false;
      foundConditionLeftOperand = false;
      // Leave foundOperator set to true, since this will always
      // be the state of the parent predicate

      // Remove this object from the stack
      queryStack.pop();
    }
    else if (localName.equals(XMLConstants.PREDICATE_ELEMENT))
    {
      // Set the state
      inPredicate = false;

      // Remove this object from the stack
      queryStack.pop();

      // Check the parent to correct set the state
      if (queryStack.isEmpty())
      {
        inPredicate = false;
      }
      else
      {
        Object parent = queryStack.peek();
        if (parent instanceof IPredicate)
        {
          inPredicate = true;
        }
      }
    }
    else if (localName.equals(XMLConstants.CRITERIA_ELEMENT))
    {
      // Set the state
      inCriteria = false;
    }
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String)
   */
  public void endPrefixMapping(String arg0) throws SAXException
  {
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int)
   */
  public void ignorableWhitespace(char[] arg0, int arg1, int arg2) throws SAXException
  {
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String, java.lang.String)
   */
  public void processingInstruction(String arg0, String arg1) throws SAXException
  {
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator)
   */
  public void setDocumentLocator(Locator arg0)
  {
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String)
   */
  public void skippedEntity(String arg0) throws SAXException
  {
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#startDocument()
   */
  public void startDocument() throws SAXException
  {
    // Reset the parser state
    queryStack = new Stack();
    inQuery = false;
    inArithmeticCondition = false;
    inFunction = false;
    inCondition = false;
    foundConditionLeftOperand = false;
    inOrderBy = false;
    inOrderByExpression = false;
    inPredicate = false;
    foundOperator = false;
    inCriteria = false;
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
   */
  public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException
  {
    if (inQuery)
    {
      // In a query - could encounter predicate, condition, orderBy, or limit
      // Could also encounter query criteria (only for a UserSelectQuery)
      if (inCriteria)
      {
        handleCriteria(localName, atts);
      }
      else if (inCondition)
      {
        handleCondition(localName, atts);
      }
      else if (inPredicate)
      {
        handlePredicate(localName, atts);
      }
      else if (inOrderBy)
      {
        handleOrderBy(localName, atts);
      }
      else if (localName.equals(XMLConstants.PREDICATE_ELEMENT))
      {
        handleQueryPredicate();
      }
      else if (localName.equals(XMLConstants.CONDITION_ELEMENT))
      {
        handleQueryCondition();
      }
      else if (localName.equals(XMLConstants.ORDER_BY_ELEMENT))
      {
        handleQueryOrderBy();
      }
      else if (localName.equals(XMLConstants.LIMIT_ELEMENT))
      {
        handleQueryLimit(atts);
      }
    }
    else if (localName.equals(XMLConstants.QUERY_ELEMENT))
    {
      handleQuery(atts);
    }
  }
  private void handleCriteria(String localName, Attributes atts)
  {
    if (localName.equals(XMLConstants.ELEMENT_ELEMENT))
    {
      String key = "";
      String value = "";

      // get the attribute values
      for (int i = 0; i < atts.getLength(); i++)
      {
        if (atts.getLocalName(i).equals(XMLConstants.NAME_ATTR))
        {
          key = atts.getValue(i);
        }
        else if (atts.getLocalName(i).equals(XMLConstants.VALUE_ATTR))
        {
          value = atts.getValue(i);
        }
      }

      // Set the element in the map
      Map criteriaMap = (Map) queryStack.peek();
      criteriaMap.put(key, value);
    }
  }

  private void handleQueryLimit(Attributes atts)
  {
    Limit limit = parseLimit(atts);

    // Set the limit on the query object
    selectQuery.setLimit(limit);
  }

  private Limit parseLimit(Attributes atts)
  {
    Limit limit = new Limit();

    // get the attribute values
    for (int i = 0; i < atts.getLength(); i++)
    {
      if (atts.getLocalName(i).equals(XMLConstants.VALUE_ATTR))
      {
        limit.setLimitValue(Integer.parseInt(atts.getValue(i)));
      }
    }
    return limit;
  }

  private void handleQueryOrderBy()
  {
    inOrderBy = true;
    OrderBy orderBy = new OrderBy();

    // Set the orderBy element on the query
    selectQuery.setOrderBy(orderBy);

    // Push the object onto the stack
    queryStack.push(orderBy);
  }

  private void handleQueryCondition()
  {
    inCondition = true;
    Condition condition = new Condition();

    // Set the condition element on the query
    selectQuery.setPredicate(condition);

    // Push the object onto the stack
    queryStack.push(condition);
  }

  private void handleQueryPredicate()
  {
    inPredicate = true;
    Predicate predicate = new Predicate();

    // Set the predicate element on the query
    selectQuery.setPredicate(predicate);

    // Push the object onto the stack
    queryStack.push(predicate);
  }

  private void handleCondition(String localName, Attributes atts) throws SAXException
  {
    // In a condition - will encounter a left operand, an operator, and then (1..m) right operands
    // Matching the left operand - will encounter arithmeticCondition, function, attribute, or value
    if (inArithmeticCondition)
    {
      handleArithmeticCondition(localName, atts);
    }
    else if (inFunction)
    {
      handleFunction(localName, atts);
    }
    else if (localName.equals(XMLConstants.OPERATOR_ELEMENT))
    {
      handleConditionOperator(atts);
    }
    else if (localName.equals(XMLConstants.ARITHMETIC_CONDITION_ELEMENT))
    {
      handleConditionArithmeticCondition(atts);
    }
    else if (localName.equals(XMLConstants.FUNCTION_ELEMENT))
    {
      handleConditionFunction(atts);
    }
    else if (localName.equals(XMLConstants.ATTRIBUTE_ELEMENT))
    {
      handleConditionAttribute(atts);
    }
    else if (localName.equals(XMLConstants.VALUE_ELEMENT))
    {
      handleConditionValue(atts);
    }
  }

  private void handleConditionValue(Attributes atts) throws SAXException
  {
    // Set the state
    if (!foundConditionLeftOperand)
    {
      foundConditionLeftOperand = true;
    }
    else if (!foundOperator)
    {
      throw new SAXException("XML Validation Error: Found " + XMLConstants.VALUE_ELEMENT + " element before " + XMLConstants.OPERATOR_ELEMENT + " element.");
    }

    Value value = parseValue(atts);

    // Get the condition from the stack to set the operator on
    Condition condition = (Condition) queryStack.peek();

    // If we haven't found the operator, we are on the left operand, else we are on the right
    if (!foundOperator)
    {
      condition.setLeftOperand(value);
    }
    else
    {
      condition.addRightOperand(value);
    }
  }
  private Value parseValue(Attributes atts)
  {
    Value value = new Value();

    // get the attribute values
    for (int i = 0; i < atts.getLength(); i++)
    {
      if (atts.getLocalName(i).equals(XMLConstants.PROPERTY_NAME_ATTR))
      {
        value.setPropertyName(atts.getValue(i));
      }
      else if (atts.getLocalName(i).equals(XMLConstants.DATA_TYPE_ATTR))
      {
        value.setDataType(Integer.parseInt(atts.getValue(i)));
      }
      else if (atts.getLocalName(i).equals(XMLConstants.VALUE_ATTR))
      {
        value.setValueString(atts.getValue(i));
      }
    }
    return value;
  }

  private void handleConditionAttribute(Attributes atts) throws SAXException
  {
    // Set the state
    if (!foundConditionLeftOperand)
    {
      foundConditionLeftOperand = true;
    }
    else if (!foundOperator)
    {
      throw new SAXException("XML Validation Error: Found " + XMLConstants.ATTRIBUTE_ELEMENT + " element before " + XMLConstants.OPERATOR_ELEMENT + " element.");
    }

    Attribute attribute = parseAttribute(atts);

    // Get the condition from the stack to set the operator on
    Condition condition = (Condition) queryStack.peek();

    // If we haven't found the operator, we are on the left operand, else we are on the right
    if (!foundOperator)
    {
      condition.setLeftOperand(attribute);
    }
    else
    {
      condition.addRightOperand(attribute);
    }
  }

  private Attribute parseAttribute(Attributes atts)
  {
	String name = null;
	int dataType = Constants.DATATYPE_UNKNOWN;
    // get the attribute values
    for (int i = 0; i < atts.getLength(); i++)
    {
      if (atts.getLocalName(i).equals(XMLConstants.NAME_ATTR))
      {
        name = atts.getValue(i);
      }
      else if (atts.getLocalName(i).equals(XMLConstants.DATA_TYPE_ATTR))
      {
        dataType = Integer.parseInt(atts.getValue(i));
      }
    }
	Attribute attribute = new Attribute(name, dataType);

    return attribute;
  }

  private void handleOrderByExpressionFunction(Attributes atts) throws SAXException
  {
    // Set the state
    inFunction = true;

    if (!foundOperator)
    {
      throw new SAXException("XML Validation Error: Found " + XMLConstants.OPERATOR_ELEMENT + " element before the orderByElement.");
    }

    Function function = parseFunction(atts);

    // Get the orderByExpression from the stack to set the orderByElement
    OrderByExpression orderByExpression = (OrderByExpression) queryStack.peek();
    orderByExpression.setOrderByElement(function);

    // Push this onto the stack
    queryStack.push(function);
  }

  private void handleConditionFunction(Attributes atts) throws SAXException
  {
    // Set the state
    inFunction = true;
    if (!foundConditionLeftOperand)
    {
      foundConditionLeftOperand = true;
    }
    else if (!foundOperator)
    {
      throw new SAXException("XML Validation Error: Found " + XMLConstants.FUNCTION_ELEMENT + " element before " + XMLConstants.OPERATOR_ELEMENT + " element.");
    }

    Function function = parseFunction(atts);

    // Get the condition from the stack to set the operator on
    Condition condition = (Condition) queryStack.peek();

    // If we haven't found the operator, we are on the left operand, else we are on the right
    if (!foundOperator)
    {
      condition.setLeftOperand(function);
    }
    else
    {
      condition.addRightOperand(function);
    }

    // Push this onto the stack
    queryStack.push(function);
  }
  private Function parseFunction(Attributes atts)
  {
    Function function = new Function();

    // get the attribute values
    for (int i = 0; i < atts.getLength(); i++)
    {
      if (atts.getLocalName(i).equals(XMLConstants.NAME_ATTR))
      {
        function.setName(atts.getValue(i));
      }
    }
    return function;
  }

  private void handleConditionArithmeticCondition(Attributes atts) throws SAXException
  {
    // Set the state
    inArithmeticCondition = true;
    if (!foundConditionLeftOperand)
    {
      foundConditionLeftOperand = true;
    }
    else if (!foundOperator)
    {
      throw new SAXException("XML Validation Error: Found " + XMLConstants.ARITHMETIC_CONDITION_ELEMENT + " element before " + XMLConstants.OPERATOR_ELEMENT + " element.");
    }

    ArithmeticCondition arithmeticCondition = parseArithmeticCondition(atts);

    // Get the condition from the stack to set the operator on
    Condition condition = (Condition) queryStack.peek();

    // If we haven't found the operator, we are on the left operand, else we are on the right
    if (!foundOperator)
    {
      condition.setLeftOperand(arithmeticCondition);
    }
    else
    {
      condition.addRightOperand(arithmeticCondition);
    }

    // Push this onto the stack
    queryStack.push(arithmeticCondition);
  }

  private void handleConditionOperator(Attributes atts) throws SAXException
  {
    // Make sure we have found the left operand
    if (!foundConditionLeftOperand)
    {
      throw new SAXException("XML Validation Error: Found " + XMLConstants.OPERATOR_ELEMENT + " element before left operand.");
    }

    // Set the state
    foundOperator = true;
    Operator operator = parseOperator(atts);

    // Get the condition from the stack to set the operator on
    Condition condition = (Condition) queryStack.peek();

    // Set the operator
    condition.setOperator(operator);
  }
  private Operator parseOperator(Attributes atts)
  {
    Operator operator = new Operator();

    // get the attribute values
    for (int i = 0; i < atts.getLength(); i++)
    {
      if (atts.getLocalName(i).equals(XMLConstants.NAME_ATTR))
      {
        operator.setName(atts.getValue(i));
      }
    }
    return operator;
  }

  private void handleFunction(String localName, Attributes atts) throws SAXException
  {
    // In an function - will encounter arithmeticCondition, function, attribute, or value
    if (localName.equals(XMLConstants.ARITHMETIC_CONDITION_ELEMENT))
    {
      // Set the state
      inFunction = false;
      inArithmeticCondition = true;

      ArithmeticCondition arithmeticCondition = parseArithmeticCondition(atts);

      // Get the function from the stack and add this to it
      Function parent = (Function) queryStack.peek();
      parent.addFunctionElement(arithmeticCondition);

      // Push this onto the stack
      queryStack.push(arithmeticCondition);
    }
    else if (localName.equals(XMLConstants.FUNCTION_ELEMENT))
    {
      Function function = parseFunction(atts);

      // Get the function from the stack and add this to it
      Function parent = (Function) queryStack.peek();
      parent.addFunctionElement(function);

      // Push this onto the stack
      queryStack.push(function);
    }
    else if (localName.equals(XMLConstants.ATTRIBUTE_ELEMENT))
    {
      Attribute attribute = parseAttribute(atts);

      // Get the function from the stack and add this to it
      Function parent = (Function) queryStack.peek();
      parent.addFunctionElement(attribute);
    }
    else if (localName.equals(XMLConstants.VALUE_ELEMENT))
    {
      Value value = parseValue(atts);

      // Get the function from the stack and add this to it
      Function parent = (Function) queryStack.peek();
      parent.addFunctionElement(value);
    }
  }

  private void handleArithmeticCondition(String localName, Attributes atts) throws SAXException
  {
    // In an arithmeticCondition - will encounter arithmeticCondition, function, attribute, or value
    if (localName.equals(XMLConstants.ARITHMETIC_CONDITION_ELEMENT))
    {
      ArithmeticCondition arithmeticCondition = parseArithmeticCondition(atts);

      // Get the arithmetic condition from the stack and add this to it
      ArithmeticCondition parent = (ArithmeticCondition) queryStack.peek();
      parent.addArithmeticConditionElement(arithmeticCondition);

      // Push this onto the stack
      queryStack.push(arithmeticCondition);
    }
    else if (localName.equals(XMLConstants.FUNCTION_ELEMENT))
    {
      // Set the state
      inArithmeticCondition = false;
      inFunction = true;

      Function function = parseFunction(atts);

      // Get the arithmetic condition from the stack and add this to it
      ArithmeticCondition parent = (ArithmeticCondition) queryStack.peek();
      parent.addArithmeticConditionElement(function);

      // Push this onto the stack
      queryStack.push(function);
    }
    else if (localName.equals(XMLConstants.ATTRIBUTE_ELEMENT))
    {
      Attribute attribute = parseAttribute(atts);

      // Get the arithmetic condition from the stack and add this to it
      ArithmeticCondition parent = (ArithmeticCondition) queryStack.peek();
      parent.addArithmeticConditionElement(attribute);

      // Push this onto the stack
      queryStack.push(attribute);
    }
    else if (localName.equals(XMLConstants.VALUE_ELEMENT))
    {
      Value value = parseValue(atts);

      // Get the arithmetic condition from the stack and add this to it
      ArithmeticCondition parent = (ArithmeticCondition) queryStack.peek();
      parent.addArithmeticConditionElement(value);

      // Push this onto the stack
      queryStack.push(value);
    }
  }

  private ArithmeticCondition parseArithmeticCondition(Attributes atts)
  {
    ArithmeticCondition arithmeticCondition = new ArithmeticCondition();

    // get the attribute values
    for (int i = 0; i < atts.getLength(); i++)
    {
      if (atts.getLocalName(i).equals(XMLConstants.OPERATION_ATTR))
      {
        arithmeticCondition.setArithmeticOperation(atts.getValue(i));
      }
    }
    return arithmeticCondition;
  }

  private void handleOrderByExpression(String localName, Attributes atts) throws SAXException
  {
    // In a orderByExpression - will encounter an operator and then an attribute or function
    if (inFunction)
    {
      handleFunction(localName, atts);
    }
    else if (localName.equals(XMLConstants.OPERATOR_ELEMENT))
    {
      // Set the state
      foundOperator = true;

      Operator operator = parseOperator(atts);

      // Get the orderByExpression from the stack and add this object
      OrderByExpression orderByExpression = (OrderByExpression) queryStack.peek();
      orderByExpression.setOperator(operator);
    }
    else if (localName.equals(XMLConstants.FUNCTION_ELEMENT))
    {
      handleOrderByExpressionFunction(atts);
    }
    else if (localName.equals(XMLConstants.ATTRIBUTE_ELEMENT))
    {
      handleOrderByExpressionAttribute(atts);
    }
  }

  private void handleOrderByExpressionAttribute(Attributes atts) throws SAXException
  {
    if (!foundOperator)
    {
      throw new SAXException("XML Validation Error: Found " + XMLConstants.OPERATOR_ELEMENT + " element before the orderByElement.");
    }

    Attribute attribute = parseAttribute(atts);

    // Get the orderByExpression from the stack and add this object
    OrderByExpression orderByExpression = (OrderByExpression) queryStack.peek();
    orderByExpression.setOrderByElement(attribute);
  }

  private void handleOrderBy(String localName, Attributes atts) throws SAXException
  {
    // In a orderBy - will encounter (1..m) order by expressions
    if (inOrderByExpression)
    {
      handleOrderByExpression(localName, atts);
    }
    else if (localName.equals(XMLConstants.ORDER_BY_EXPRESSION_ELEMENT))
    {
      inOrderByExpression = true;

      OrderByExpression orderByExpression = new OrderByExpression();

      // Get the orderBy from the stack and add this object
      OrderBy orderBy = (OrderBy) queryStack.peek();
      orderBy.addOrderByExpression(orderByExpression);

      // Push this object on the stack
      queryStack.push(orderByExpression);
    }
  }

  private void handlePredicate(String localName, Attributes atts) throws SAXException
  {
    // In a predicate - will encounter operator, then (1..m) predicates or conditions
    if (localName.equals(XMLConstants.OPERATOR_ELEMENT))
    {
      foundOperator = true;
      Operator operator = parseOperator(atts);

      // Get the predicate from the stack to set the operator on
      Predicate predicate = (Predicate) queryStack.peek();
      predicate.setOperator(operator);
    }
    else if (localName.equals(XMLConstants.PREDICATE_ELEMENT))
    {
      if (foundOperator)
      {
        // Set the state
        foundOperator = false;
        Predicate predicate = new Predicate();
        // Add the predicate to the parent on the stack (must be a predicate)
        Predicate parent = (Predicate) queryStack.peek();
        parent.addPredicateBase(predicate);
        // Push this onto the stack
        queryStack.push(predicate);
      }
      else
      {
        throw new SAXException("XML Validation Error: Found " + XMLConstants.PREDICATE_ELEMENT + " element before " + XMLConstants.OPERATOR_ELEMENT + " element.");
      }
    }
    else if (localName.equals(XMLConstants.CONDITION_ELEMENT))
    {
      if (foundOperator)
      {
        // Set the state
        inCondition = true;
        foundOperator = false;
        Condition condition = new Condition();
        // Add the condition to the parent on the stack (must be a predicate)
        Predicate parent = (Predicate) queryStack.peek();
        parent.addPredicateBase(condition);
        // Push the object onto the stack
        queryStack.push(condition);
      }
      else
      {
        throw new SAXException("XML Validation Error: Found " + XMLConstants.CONDITION_ELEMENT + " element before " + XMLConstants.OPERATOR_ELEMENT + " element.");
      }
    }
  }

  private void handleQuery(Attributes atts) throws SAXException
  { // We are in the query element
    inQuery = true; // get the attribute values
    for (int i = 0; i < atts.getLength(); i++)
    {
      if (atts.getLocalName(i).equals(XMLConstants.CLASS_ATTR))
      { // Create this class
        try
        {
          selectQuery = (ISelectQuery) Class.forName(atts.getValue(i)).newInstance();
        }
        catch (Exception e)
        {
          throw new SAXException(e.getMessage());
        }
      }
    }
  }

  /* (non-Javadoc)
   * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String, java.lang.String)
   */
  public void startPrefixMapping(String arg0, String arg1) throws SAXException
  {
  }

  /**
   * @return
   */
  public ISelectQuery getSelectQuery()
  {
    return selectQuery;
  }

  /**
   * @param query
   */
  public void setSelectQuery(ISelectQuery query)
  {
    selectQuery = query;
  }
}