/*
 * SimpleValue.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.util.Calendar;
import java.util.Date;

/**
 * A <code>DoubleValue</code> provides an implementation
 * of the <code>Value</code> interface representing a double value.
 *
 * @author Stefan Guggisberg
 */
public class DoubleValue extends BaseValue {

    public static final int TYPE = PropertyType.DOUBLE;

    private Double dblNumber = null;

    /**
     * Constructs a <code>DoubleValue</code> object representing a double.
     *
     * @param dblNumber the double this <code>DoubleValue</code> should represent
     */
    public DoubleValue(Double dblNumber) {
        super(TYPE);
        this.dblNumber = dblNumber;
    }

    /**
     * Constructs a <code>DoubleValue</code> object representing a double.
     *
     * @param dbl the double this <code>DoubleValue</code> should represent
     */
    public DoubleValue(double dbl) {
        super(TYPE);
        this.dblNumber = new Double(dbl);
    }

    /**
     * Returns a new <code>DoubleValue</code> initialized to the value
     * represented by the specified <code>String</code>.
     * @param s the string to be parsed.
     * @return a newly constructed <code>DoubleValue</code> representing the
     *         the specified value.
     * @throws ValueFormatException If the <code>String</code> does not
     *         contain a parsable <code>double</code>.
     */
    public static DoubleValue valueOf(String s) throws ValueFormatException {
        try {
            return new DoubleValue(Double.parseDouble(s));
        } catch (NumberFormatException e) {
            throw new ValueFormatException("invalid format", e);
        }
    }

    //----------------------------------------------------------------< Value >
    /**
     * @see Value#getDate
     */
    public Calendar getDate() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (dblNumber != null) {
            // loosing timezone information...
            Calendar cal = Calendar.getInstance();
            cal.setTime(new Date(dblNumber.longValue()));
            return cal;
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getLong
     */
    public long getLong() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (dblNumber != null) {
            return dblNumber.longValue();
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getBoolean
     */
    public boolean getBoolean() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to boolean failed: inconvertible types");
    }

    /**
     * @see Value#getDouble
     */
    public double getDouble() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (dblNumber != null) {
            return dblNumber.doubleValue();
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getString
     */
    public String getString() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (dblNumber != null) {
            return dblNumber.toString();
        } else {
            throw new ValueFormatException("empty value");
        }
    }
    
    
  public boolean equals(Object obj)
  {
    boolean equals = false;
    if (obj instanceof DoubleValue)
    {
      DoubleValue doubleValue = (DoubleValue)obj;
      if (doubleValue.dblNumber != null && dblNumber != null)
      {
        equals = doubleValue.dblNumber.equals(dblNumber);
      }
      else if (doubleValue.dblNumber == null && dblNumber == null)
      {
        equals = true;
      }
    }
    return equals;
  }
  
  public int hashCode()
  {
    int hashCode = 37;
    if (dblNumber != null)
    {
      hashCode = dblNumber.hashCode();
    }
    return hashCode;
  }
}
