/*
 * RepositoryException.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * Main exception thrown by classes in this package. May either contain
 * an error message or another exception wrapped inside this exception.
 *
 * @author Peeter Piegaze
 */
public class RepositoryException extends Exception {

    /** Root failure cause */
    protected Exception rootException;

    /**
     * Class constructor that creates a <code>RepositoryException</code>
     */
    public RepositoryException() {
        super();
    }

    /**
     * Class constructor that creates a <code>RepositoryException</code> given
     * a message describing the failure cause
     * @param s description
     */
    public RepositoryException(String s) {
        super(s);
    }

    /**
     * Class constructor that creates a <code>RepositoryException</code> given
     * a message describing the failure and a root exception
     * @param s description
     * @param e root failure cause
     */
    public RepositoryException(String s, Exception e) {
        super(s);

        /* don't create a proxy to a proxy */
        if (e instanceof RepositoryException) {
            RepositoryException e2 = (RepositoryException) e;
            rootException = e2.rootException;
        } else {
            rootException = e;
        }
    }

    /**
     * Class constructor that creates a <code>RepositoryException</code> given
     * a root exception
     * @param e root failure cause
     */
    public RepositoryException(Exception e) {
        this(null, e);
    }

    /**
     * Returns the error message string of this exception.
     * @return error message string of this exception.
     */
    public String getMessage() {
        String s = super.getMessage();
        if (rootException == null) {
            return s;
        } else {
            String s2 = rootException.getMessage();
            return s == null ? s2 : s + ": " + s2;
        }
    }

    /**
     * Returns the root exception of this exception.
     * @return the root exception of this exception
     */
    public Exception getRootException() {
        return rootException;
    }

    /**
     * Prints this <code>RepositoryException</code> and its backtrace to the
     * standard error stream.
     */
    public void printStackTrace() {
        synchronized (System.err) {
            super.printStackTrace();
            if (rootException != null) {
                rootException.printStackTrace();
            }
        }
    }

    /**
     * Prints this <code>RepositoryException</code> and its backtrace to the
     * specified print stream.
     * @param s <code>PrintStream</code> to use for output
     */
    public void printStackTrace(PrintStream s) {
        synchronized (s) {
            super.printStackTrace(s);
            if (rootException != null) {
                rootException.printStackTrace(s);
            }
        }
    }

    /**
     * Prints this <code>RepositoryException</code> and its backtrace to
     * the specified print writer.
     * @param s <code>PrintWriter</code> to use for output
     * @since JDK1.1
     */
    public void printStackTrace(PrintWriter s) {
        synchronized (s) {
            super.printStackTrace(s);
            if (rootException != null) {
                rootException.printStackTrace(s);
            }
        }
    }
}
