/*
 * Workspace.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.io.InputStream;
import java.io.OutputStream;

import javax.jcr.access.AccessDeniedException;
import javax.jcr.access.AccessManager;
import javax.jcr.lock.LockCapabilities;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.query.QueryManager;

import org.xml.sax.ContentHandler;

/**
 * Represents a view onto the the content repository
 * associated with a particular <code>Ticket</code> .
 * Acquired by calling <code>{@link Repository#getWorkspace(Ticket)}</code>.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface Workspace {

    /**
     * Static type used for workspace type.
     */
    public static final class WorkspaceType
    {
      public static final WorkspaceType STABLE = new WorkspaceType();
      public static final WorkspaceType DYNAMIC = new WorkspaceType();
      
      private WorkspaceType() { }
    }
   
    /**
     * Returns the name of this workspace.
     * 
     * @return The workspace name.
     */
    public String getName();

    /**
     * Returns the type of this workspace.
     * 
     * @return A WorkspaceType indicating the type of this workspace.
     */
    public WorkspaceType getType();
    
    /**
     * Gets the ticket that was used to aqcuire this <code>Workspace</code>
     * object.
     * Returns the <code>Ticket</code> object through which this Workspace
     * object was acquired.
     *
     * @return a <code>{@link Ticket}</code> object.
     */
    public Ticket getTicket();

    /**
     * Returns the root node of the repository.
     * The root node, "/", is the main access point to the content of the
     * repository.
     *
     * @return The root node of the repository: a <code>Node</code> object.
     *
     * @throws RepositoryException if an error occurs.
     */
    public Node getRootNode() throws RepositoryException;

    /**
     * Returns the node specifed by the given UUID.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * The UUID will be specific to the repository implementation
     * (so, technically it may only be an ID not a UUID) not be. In fact the
     * repository may not even support UUIDs, in which case this method will throw
     * <code>UnsupportedRepositoryOperationException</code>. If UUIDs are supported
     * but the particular UUID passed cannot be found then an
     * <code>ItemNotFoundException</code> is thrown.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * For those nodes that are of node type <code>nt:hardlinkable</code>
     * (which exposes a UUID as a property) this method will use that UUID.
     * If no node exists with the given UUID an ItemNotFoundException is
     * thrown.
     *
     * @param Uuid A universally unique identifier.
     *
     * @return A <code>Node</code>.
     *
     * @throws UnsupportedRepositoryOperationException if the repository does
     * not support UUIDs.
     * @throws ItemNotFoundException if the specified UUID is not found.
     * @throws RepositoryException if another error occurs.
     */
    public Node getNodeByUuid(String Uuid) throws UnsupportedRepositoryOperationException, ItemNotFoundException, RepositoryException;

    /**
     * Returns the node found at the absolute path specified.
     * 
     * @param absPath The absolute path of a node in this workspace.
     * 
     * @return The node at the specified path.
     * 
     * @throws AccessDeniedException if the ticket used to access this workspace has insufficient authority.
     * @throws ItemNotFoundException if there is no node at absPath in the workspace.
     * @throws RepositoryException if another error occurs.
     */
    public Node getNodeByAbsPath(String absPath) throws AccessDeniedException, ItemNotFoundException, RepositoryException;
    
    /**
     * Returns the property found at the absolute path specified.
     * 
     * @param absPath The absolute path of a property in this workspace.
     * 
     * @return The property at the specified path.
     * 
     * @throws AccessDeniedException if the ticket used to access this workspace has insufficient authority.
     * @throws ItemNotFoundException if there is no property at absPath in the workspace.
     * @throws RepositoryException if another error occurs.
     */
    public Property getPropertyByAbsPath(String absPath) throws AccessDeniedException, ItemNotFoundException, RepositoryException;
  
    /**
     * Saves the state of all nodes and properties that have
     * been changed in this workspace since the last <code>save</code>,
     * <code>revert</code>, or <code>get</code>.
     * 
     * @throws ConstraintViolationException if node type validation fails.
     * @throws AccessDeniedException if the ticket used to access this workspace has insufficient authority.
     * @throws StaleItemException if the item being saved has been modified by another.
     * user since this user retrieved it.
     * @throws RepositoryException if another error occurs.
     */
//  public void save() throws ConstraintViolationException, AccessDeniedException, StaleItemException;
    public void save() throws AccessDeniedException, ConstraintViolationException, StaleValueException, RepositoryException;

    /**
     * Discards all changes made to nodes and properties since
     * the last <code>save</code> or <code>get</code> of those
     * items.
     * 
     * @throws AccessDeniedException if the ticket used to access this workspace has insufficient authority.
     * @throws RepositoryException if another error occurs.
     */
    public void revert() throws AccessDeniedException, RepositoryException;
    
    /**
     * Copies the node at <code>srcPath</code> to the new location at
     * <code>destPath</code>.
     * If <code>shallow</code> is true then only the node at <code>srcPath</code>
     * and its properties (but not its child nodes) is copied to the new
     * location, otherwise its entire subtree is copied. Throws an exception if
     * there is already an item at <code>destPath</code>, or if some other
     * constraint (access, node type, system-level) is violated. If succesful,
     * the parent of the new node at <code>destPath</code> is "saved" (i.e. the
     * change is persisted immediately, there is no need to call
     * <code>Node.save()</code>).
     *
     * @param srcPath the path of the node to be copied.
     * @param destPath the location to which the node at <code>srcPath</code>
     * is to be copied.
     * @param shallow if true only the srcnode and its properties are copied,
     * otherwise the source node and its entire subtree is copied.
     * @throws ConstraintViolationException if the copy would violate a
     * node type or other constraint.
     * @throws AccessDeniedException if the current ticket (i.e. the ticket that
     * was used to aqcuire this <code>Workspace</code> object) does not have
     * sufficient access rights to complete the operation.
     * @throws PathNotFoundException if the node at <code>srcPath</code> or
     * the parent of the new node at <code>destPath</code> does not exist.
     * @throws ItemExistsException if a node or property already exists at
     * <code>destPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public void copy(String srcPath, String destPath, boolean shallow) throws ConstraintViolationException, AccessDeniedException, PathNotFoundException, ItemExistsException, RepositoryException;

    /**
     * Copies the node at <code>srcPath</code> to the new location at
     * <code>destPath</code>.
     * Equivalent to <code>copy(srcPath, destPath, false)</code>.
     *
     * @param srcPath the path of the node to be copied.
     * @param destPath the location to which the node at <code>srcPath</code>
     * is to be copied.
     * @throws ConstraintViolationException if the copy would violate a
     * node type or other constraint.
     * @throws AccessDeniedException if the current ticket (i.e. the ticket that
     * was used to aqcuire this <code>Workspace</code> object) does not have
     * sufficient access rights to complete the operation.
     * @throws PathNotFoundException if the node at <code>srcPath</code> or
     * the parent of the new node at <code>destPath</code> does not exist.
     * @throws ItemExistsException if a node or property already exists at
     * <code>destPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public void copy(String srcPath, String destPath) throws ConstraintViolationException, AccessDeniedException, PathNotFoundException, ItemExistsException, RepositoryException;

    /**
     * Moves the node at <code>srcPath</code> (and its entire subtree) to the
     * new location at <code>destPath</code>.
     * Throws an exception if
     * there is already an item at <code>destPath</code>, or if some other
     * constraint (access, node type, system-level) is violated. If succesful,
     * the parent of the new node at <code>destPath</code> is "saved" (i.e. the
     * change is persisted immediately as is the parent of the moved node at its
     * original location, there is no need to call <code>Node.save()</code>).
     *
     * @param srcPath the path of the node to be moved.
     * @param destPath the location to which the node at <code>srcPath</code>
     * is to be moved.
     * @throws ConstraintViolationException if the move would violate a
     * node type or other constraint.
     * @throws AccessDeniedException if the current ticket (i.e. the ticket that
     * was used to aqcuire this <code>Workspace</code> object) does not have
     * sufficient access rights to complete the operation.
     * @throws PathNotFoundException if the node at <code>srcPath</code> or
     * the parent of the new node at <code>destPath</code> does not exist.
     * @throws ItemExistsException if a node or property already exists at
     * <code>destPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public void move(String srcPath, String destPath) throws ConstraintViolationException, AccessDeniedException, PathNotFoundException, ItemExistsException, RepositoryException;

    /**
     * Gets the <code>NodeTypeManager</code> object.
     * Returns the <code>NodeTypeManager</code> through which node type
     * information can be queried.
     *
     * @return a <code>NodeTypeManager</code> object.
     */
    public NodeTypeManager getNodeTypeManager();

    /**
     * Serializes the subtree rooted at the node at <code>absPath</code>
     * (it must be a node, not a property) as a stream of SAX
     * events to the supplied <code>org.xml.sax.ContentHandler</code>.
     * The XML represented by these SAX events will be in the
     * <b>system view</b> form.
     * <p>
     * If <code>binaryAsLink</code> is <code>true</code> then any properties
     * of <code>PropertyType.BINARY</code> will be replaced in the serialized
     * output by properties of <code>PropertyType.SOFTLINK</code> recording
     * their own absolute path.
     * <p>
     * If <code>binaryAsLink</code> is <code>false</code> then the actual value
     * of each <code>BINARY</code> property is recorded using Base64 encoding.
     * <p>
     * If <code>noRecurse</code> is true then only the node at
     * <code>absPath</code> and its properties, but not its child nodes, is
     * serialized.
     * <p>
     * If <code>noRecurse</code> is <code>false</code> then the entire subtree
     * rooted at <code>absPath</code> is serialized.
     * <p>
     * If the user lacks read access to some subsection of the specified tree,
     * that section simply does not get serialized, since, from the user's
     * point of view, it is not there.
     *
     * @param absPath The path of the root of the subtree to be serialized.
     * This must be the path to a node, not a property
     * @param contentHandler The  <code>org.xml.ContentHandler</code> to
     * which the SAX events representing the XML serialization of the subtree
     * will be output.
     * @param binaryAsLink A <code>boolean</code> governing whether binary
     * properties are to converted to soft links.
     * @param noRecurse A <code>boolean</code> governing whether the subtree at
     * absPath is to be recursed.
     *
     * @throws PathNotFoundException if no node exists at
     * <code>absPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public void exportSysView(String absPath, ContentHandler contentHandler, boolean binaryAsLink, boolean noRecurse) throws PathNotFoundException,  RepositoryException;

    /**
     * Serializes the subtree rooted at the node (it must be a node, not a
     * property)at <code>absPath</code> as an  XML document and streams it to
     * the supplied <code>OutputStream</code>. The resulting XML is in the
     * <b>system view</b> form.
     * <p>
     * If <code>binaryAsLink</code> is <code>true</code> then any properties
     * of <code>PropertyType.BINARY</code> will be replaced in the serialized
     * output by properties of <code>PropertyType.SOFTLINK</code> recording
     * their own absolute path.
     * <p>
     * If <code>binaryAsLink</code> is <code>false</code> then the actual value
     * of each <code>BINARY</code> property is recorded using Base64 encoding.
     * <p>
     * If <code>noRecurse</code> is true then only the node at
     * <code>absPath</code> and its properties, but not its child nodes, is
     * serialized.
     * <p>
     * If <code>noRecurse</code> is <code>false</code> then the entire subtree
     * rooted at <code>absPath</code> is serialized.
     * <p>
     * If the user lacks read access to some subsection of the specified tree,
     * that section simply does not get serialized, since, from the user's
     * point of view, it is not there.
     *
     * @param absPath The path of the root of the subtree to be serialized.
     * @param out The <code>OutputStream</code> to which the XML
     * serialization of the subtree will be output.
     * @param binaryAsLink A <code>boolean</code> governing whether binary
     * properties are to converted to soft links.
     * @param noRecurse A <code>boolean</code> governing whether the subtree at
     * <code>absPath</code> is to be recursed.
     *
     * @throws PathNotFoundException if no node exists at
     * <code>absPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public void exportSysView(String absPath, OutputStream out, boolean binaryAsLink, boolean noRecurse) throws PathNotFoundException,  RepositoryException;

    /**
     * Same as <code>exportSysView(String absPath, ContentHandler contentHandler,
     * boolean binaryAsLink, boolean noRecurse)</code> except that the resulting
     * XML is in the <b>document view</b> form.
     *
     * @param absPath The path of the root of the subtree to be serialized.
     * @param contentHandler The  <code>org.xml.ContentHandler</code> to
     * which the SAX events representing the XML serialization of the subtree
     * will be output.
     * @param binaryAsLink A <code>boolean</code> governing whether binary
     * properties are to converted to soft links.
     * @param noRecurse A <code>boolean</code> governing whether the subtree at
     * <code>absPath</code> is to be recursed.
     *
     * @throws PathNotFoundException if no node exists at
     * <code>absPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public void exportDocView(String absPath, ContentHandler contentHandler, boolean binaryAsLink, boolean noRecurse) throws PathNotFoundException,  RepositoryException;

    /**
     * Same as <code>exportSysView(String absPath, OutputStream out,
     * boolean binaryAsLink, boolean noRecurse</code> except that the resulting
     * XML is in the <b>document view</b> form.
     *
     * @param absPath The path of the root of the subtree to be serialized.
     * @param out The <code>OutputStream</code> to which the XML
     * serialization of the subtree will be output.
     * @param binaryAsLink A <code>boolean</code> governing whether binary
     * properties are to converted to soft links.
     * @param noRecurse A <code>boolean</code> governing whether the subtree at
     * <code>absPath</code> is to be recursed.
     *
     * @throws PathNotFoundException if no node exists at
     * <code>absPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public void exportDocView(String absPath, OutputStream out, boolean binaryAsLink, boolean noRecurse) throws PathNotFoundException,  RepositoryException;

    /**
     * Deserialize an XML document (in <b>system view</b> form) and add the
     * resulting item subtree as a child of the node at <code>parentPath</code>.
     * <p>
     * If node type restrictions prevent the addition of the subtree to the
     * node at <code>parentPath</code>, a <code>ConstraintViolationException</code>
     * is thrown.
     * <p>
     * If the XML stream provided is not a valid JCR <b>system view</b> XML
     * document then an <code>InvalidSerializedDataException</code> is thrown.
     * <p>
     * If the user does not have sufficient access rights to write the
     * deserialized nodes and properties, then an
     * <code>AccessDeniedException</code> is thrown.
     * <p>
     * In level 2: The deserialization mechanism must take into account the
     * property <code>jcr:uuid</code> in order to reconstruct hard links
     * across serialization/deserialization cycles. Throws
     * <code>ConstraintViolationException</code> on node type mismatch due to
     * hard links.
     *
     * @param parentPath the absolute path of the node below which the deserialized
     * subtree is added.
     * @param in The <code>Inputstream</code> from which the XML to be deserilaized
     * is read.
     *
     * @throws PathNotFoundException if no node exists at <code>parentPath</code>.
     * @throws ConstraintViolationException if a node type restriction prevents
     * the building of the deserialized subtree.
     * @throws InvalidSerializedDataException if the seriliazed data being input is
     * not a valid JCR <b>system view</b> XML document.
     * @throws AccessDeniedException if access restrictions prevent the building of the
     * deserialized subtree.
     * @throws RepositoryException is another error occurs.
     */
    public void importSysView(String parentPath, InputStream in) throws PathNotFoundException, ConstraintViolationException, InvalidSerializedDataException, AccessDeniedException, RepositoryException;

    /**
     * Deserializes the stream of SAX events defined by the series of calls of
     * the methods of the <code>org.xml.ContentHandler</code> into a subtree of
     * items immediately below the node at <code>parentPath</code>. If the
     * incoming XML stream (in the form of SAX events) is not a valid JCR
     * system view XML document then the <code>ContentHandler</code> will throw
     * a SAXException in one of its methods.
     *
     * @param parentPath the absolute path of a node under which (as child) the imported
     * subtree will be built.
     *
     * @return an org.xml.ContentHandler whose methods may be called to feed SAX
     * events into the deserializer.
     *
     * @throws PathNotFoundException if no node exists at <code>parentPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public ContentHandler importSysView(String parentPath) throws PathNotFoundException, RepositoryException;

    /**
     * Same as <code>importSysView(String parentPath, InputStream in)</code> except the source XML must be in
     * document view form.
     * <p>
     * <b>Support of this method is optional</b>.
     * <p>
     * In implementations that do not support this method, it will throw an
     * <code>UnsupportedRepositoryOperationException</code>.
     *
     * @param parentPath the absolute path of the node below which the deserialized
     * subtree is added.
     * @param in The <code>Inputstream</code> from which the XML to be deserilaized
     * is read.
     *
     * @throws PathNotFoundException if no node exists at <code>parentPath</code>.
     * @throws ConstraintViolationException if a node type restriction prevents
     * the building of the deserialized subtree.
     * @throws InvalidSerializedDataException if the seriliazed data being input is
     * not a valid JCR <b>document view</b> XML document.
     * @throws AccessDeniedException if access restrictions prevent the building of the
     * deserialized subtree.
     * @throws RepositoryException is another error occurs.
     */
    public void importDocView(String parentPath, InputStream in) throws PathNotFoundException, ConstraintViolationException, InvalidSerializedDataException, AccessDeniedException, UnsupportedRepositoryOperationException, RepositoryException;

    /**
     * Same as <code>importSysView(String parentPath)</code> except that the
     * source XML must be in <b>document view</b> form.
     * <p>
     * <b>Support of this method is optional</b>.
     * <p>
     * In implementations that do not support this method, it will throw an
     * <code>UnsupportedRepositoryOperationException</code>.
     *
     * @param parentPath the absolute path of a node under which (as child) the imported
     * subtree will be built.
     *
     * @return an org.xml.ContentHandler whose methods may be called to feed SAX
     * events into the deserializer.
     *
     * @throws PathNotFoundException if no node exists at <code>parentPath</code>.
     * @throws RepositoryException if another error occurs.
     */
    public ContentHandler importDocView(String parentPath) throws PathNotFoundException, UnsupportedRepositoryOperationException, RepositoryException;

    /**
     * Gets the <code>QueryManager</code>.
     * Gets the <code>QueryManager</code> through which searching is done.
     *
     * @return the <code>QueryManager</code> object.
     */
    public QueryManager getQueryManager();

    /**
     * Gets the <code>NamespaceRegistry</code>.
     * Gets the <code>NamespaceRegistry</code> through which namespace operations are done
     * @return the namespace registry
     */
    public NamespaceRegistry getNamespaceRegistry();

    /**
     * In level 2 gets the <code>AccessManager</code>, in level 1 throws
     * an <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Always throws an <code>UnsupportedRepositoryOperationException</code> since
     * access control is not supported in level 1.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * Gets the <code>AccessManager</code> through which access control
     * information is queried.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: Never.
     *
     * @return an <code>AccessManager</code> object.
     */
    public AccessManager getAccessManager() throws UnsupportedRepositoryOperationException;

    /**
     * <i>If locking is supported</i> returns the
     * <code>LockCapabilities</code> object, otherwise throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Always throws an <code>UnsupportedRepositoryOperationException</code>
     * since locking is not supported in level 1.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * If locking is supported, returns the <code>LockCapabilities</code>
     * object through which dynamic discovery of supported locking functionality
     * is provided. If locking is not supported then this method throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: if implementation does not support locking.
     *
     * @return A <code>LockCapabilities</code> object.
     */
    public LockCapabilities getLockCapabilities() throws UnsupportedRepositoryOperationException;
}
