package javax.jcr.version;

import java.util.Calendar;

import javax.jcr.RepositoryException;

public interface Version
{
  /**
   * Returns the version name of this version. This corresponds to the
   * value of the <code>jcr:versionName</code> property in the
   * <code>nt:version</code> node that represents this version.
   *
   * @return String
   */
        public String getVersionName();

  /**
   * Returns the version date of this version. This corresponds to the
   * value of the <code>jcr:versionDate</code> property in the
   * <code>nt:version</code> node that represents this version.
   *
   * @return Calendar
   */   
        public Calendar getVersionDate();
        
  /**
   * Returns the version labels of this version. This corresponds to
   * the values of the <code>jcr:versionName</code> property in the
   * <code>nt:version</code> node that represents this version.
   *
         * @return String array of version labels
         */
        public String[] getVersionLabels();
        
  /**
   * Returns the version labels of this version. This corresponds to
   * the values of the <code>jcr:versionName</code> multi-value property
   * in the <code>nt:version</code> node that represents this version.
   *
   * @param label
   */
        public void addVersionLabel(String label) throws RepositoryException, VersionException;
        
  /**
   * Removes the specified label from among the labels of this version.
   * This corresponds to removing a value from the <code>jcr:versionLabel</code>
   * multi-value property in the <code>nt:version</code> node that represents
   * this version.
   *
   * @param label
   */
        public void removeVersionLabel(String label);
        
  /**
   * Returns the successor versions of this version. This corresponds to returning
   * all the <code>nt:version</code> nodes referenced by the <code>jcr:successors</code>
   * multi-value property in the <code>nt:version</code> node that represents this version.
   *
   * @return Version array of successor versions, if any
   */
        public Version[] getSuccessors();
        
  /**
   * Returns the predecessor versions of this version. This corresponds to returning all
   * the <code>nt:version</code> nodes whose <code>jcr:successors</code> property include
   * a reference to the <code>nt:version</code> node that represents this version.
   *
   * @return Version array of predecessor version or versions
   */
        public Version[] getPredecessors();
        
  /**
   * Adds the specified version <code>v</code> as a successor of this version. This is used to create a
   * merge within the version graph (not to be confused with the <code>Node.merge</code> method which
   * operates on workspace nodes). A workspace merge may be used to produce the appropriate
   * node that can then be first checked-in and then added as a successor to more than one
   * existing version, thus effecting both the semantic and the version graph parts of the
   * merge operation. This method corresponds to adding a reference to an <code>nt:version</code> node
   * to the <code>jcr:successors</code> multi-value property of the <code>nt:version</code> node that represents this
   * version. If <code>v</code> is not already in the same version history as this node or if adding <code>v</code> as
   * a successor would create a cycle in the version history then an <code>InvalidVersionException</code> is thrown.
   *
   * @param v
   * @throws InvalidVersionException
   */
        public void addSuccessor(Version v) throws InvalidVersionException;
        
  /**
   * Removes <code>v</code> from the successors of this version. This method corresponds to removing a
   * reference to an <code>nt:version</code> node from the <code>jcr:successors</code> multi-value property of the
   * <code>nt:version</code> node that represents this version. If <code>v</code> is not currently a direct successor
   * of this node then an <code>InvalidVersionException</code> is thrown.
   *
   * @param v
   * @throws InvalidVersionException
   */
        public void removeSuccessor(Version v) throws InvalidVersionException;
        
        
}
